import lib.wmainfo
from xl import xlmisc

TYPE = 'wma'

TAG_TRANSLATION = {
    "Author":       "artist",
    "AlbumTitle":   "album",
    "Title":        "title",
    "Genre":        "genre",
    "TrackNumber":  "tracknumber",
    "Year":         "date"
}

def get_tag(inf, name):
    if inf.tags.has_key(name):
        return [inf.tags[name]]
    else:
        return []

def can_change(tag):
    return False

def is_multi():
    # FIXME: is this true?
    return False

def fill_tag_from_path(tr):
    try:
        inf = lib.wmainfo.WmaInfo(tr.io_loc)
    except:
        xlmisc.log("Couldn't read tags from file: " + tr.loc)
        return

    tr.length = inf.info["playtime_seconds"]
    tr.bitrate = inf.info["max_bitrate"]

    for wma_tag, tag in TAG_TRANSLATION.iteritems():
        tr.tags[tag] = get_tag(inf, wma_tag)




#!/usr/bin/env python
# Copyright (C) 2006 Andras Petrik <bikmak@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import gtk, gobject, pygtk
from gettext import gettext as _
import xl.plugins as plugins

PLUGIN_NAME = _("Tray Buttons")
PLUGIN_AUTHORS = ['Andras Petrik <bikmak@gmail.com>']
PLUGIN_VERSION = '0.7.3'
PLUGIN_DESCRIPTION = _(r"""This plugin adds "Play/Pause", "Previous" and "Next" buttons to the notification area""")
PLUGIN_ENABLED = False
PLUGIN_ICON = None

PLUGIN = None

CONNS = plugins.SignalContainer()

class Plugin:
	
	"""
		It represents the class for the plugin
	"""
	
	
	def __init__(self, app):
		
		self.app = app
		self.buttonReverse = app.settings.get_boolean( 'button_reverse', plugin=plugins.name(__file__), default = False )
		
		if ( self.buttonReverse == False ) :
			self.tyPrevious = gtk.status_icon_new_from_stock( "gtk-media-previous" )
			self.tyPause = gtk.status_icon_new_from_stock( "gtk-media-pause" )
			self.tyNext = gtk.status_icon_new_from_stock( "gtk-media-next" )
		else :
			self.tyNext = gtk.status_icon_new_from_stock( "gtk-media-next" )
			self.tyPause = gtk.status_icon_new_from_stock( "gtk-media-pause" )
			self.tyPrevious = gtk.status_icon_new_from_stock( "gtk-media-previous" )
			
		
		self.tyPrevious.connect( "activate", self.onPrevious, None )
		self.tyPrevious.set_tooltip( _("Previous Track") )
		
		self.tyPause.connect( "activate", self.onPause, None )
		self.tyPause.set_tooltip( _("Play/Pause") )
		
		self.tyNext.connect( "activate", self.onNext, None )
		self.tyNext.set_tooltip( _("Next Track") )	
		
		self.tyPrevious.set_visible( app.settings.get_boolean( 'previous_visible', plugin=plugins.name( __file__ ), default = True ) )
		self.tyPause.set_visible( app.settings.get_boolean( 'pause_visible', plugin=plugins.name( __file__ ), default = True ) )
		self.tyNext.set_visible( app.settings.get_boolean( 'next_visible', plugin=plugins.name( __file__ ), default = True ) )
		self.showTrackInformation = app.settings.get_boolean( 'show_track_information', plugin=plugins.name(__file__), default = True )
	
	def onPause( self, par1, par2 ):
		"""
			Called when toggle pause icon is clicked
		"""
		self.app.player.toggle_pause()
		self.setPauseIcon()

	def onPrevious( self, par1, par2 ):
		"""
			Called when previous icon is clicked
		"""
		self.app.player.previous()
	
	def onNext( self, par1, par2 ):
		"""
			Called when next icon is clicked
		"""
		self.app.player.next()
		
	def destroy( self ):
		"""
			destroy the object?! ???
		"""
		
		self.tyPause.set_visible( False )
		self.tyPrevious.set_visible( False )
		self.tyNext.set_visible( False )
		
	def setPauseVisible( self, visible ) :
		"""
			set the visibility of the pause icon
		"""
		self.tyPause.set_visible( visible )
	
	def setPreviousVisible( self, visible ) :
		"""
			set the visibility of the previous icon
		"""
		self.tyPrevious.set_visible( visible )
		
	def setNextVisible( self, visible ) :
		"""
			set the visibility of the next icon
		"""
		self.tyNext.set_visible( visible )
		
	def setPauseIcon( self ) :
		"""
			Set the picture of the play/pause icon
		"""
		if APP.player.is_paused():
			self.tyPause.set_from_stock( "gtk-media-play" )
			self.tyPause.set_tooltip( _("Play") )
		else:
			self.tyPause.set_from_stock( "gtk-media-pause" )
			self.tyPause.set_tooltip( _("Pause") )
		
	def setShowTrackInformation( self, inp ) :
		"""
			set the showTrackInformation variable
		"""
		self.showTrackInformation = inp
		if ( inp == False ) :
			self.tyPrevious.set_tooltip( _("Previous Track") )
			self.tyNext.set_tooltip( _("Next Track") )
		
	def getShowTrackInformation( self ) :
		"""
			return with the showTrackInformation variable's value
		"""
		return self.showTrackInformation
		
	def setNextTooltip( self, current ) :
		"""
			Set the tooltip of the next icon
		"""
		track = self.app.tracks.get_previous_track( current )
		if track:
			self.tyPrevious.set_tooltip(_("Previous track: %(title)s by %(artist)s") % {
				'title' : track.title,
				'artist' : track.artist
			})
		else:
			self.tyPrevious.set_tooltip("")
		
	def setPreviousTooltip( self, current ) :
		"""
			Set the tooltip of the previous icon
		"""
		track = self.app.tracks.get_next_track( current )
		if track:
			self.tyNext.set_tooltip(_("Next track: %(title)s by %(artist)s") % {
				'title' : track.title,
				'artist' : track.artist
			})
		else:
			self.tyNext.set_tooltip("")



def initialize():
	"""
		Called when the plugin is enabled
	"""
	global APP, PLUGIN
	if ( PLUGIN == None ) :
		PLUGIN = Plugin( APP )
	else :
		PLUGIN.setPauseVisible( True )
		PLUGIN.setPreviousVisible( True )
		PLUGIN.setNextVisible( True )
	CONNS.connect( APP.player, 'pause-toggled', pause_toggled )
	CONNS.connect( APP.player, 'play-track', play_track )
	return True

def destroy():
	"""
		Called when the plugin is disabled
	"""
	
	global PLUGIN
	PLUGIN.destroy()
	
def configure():
	"""
		Configuration Window
	"""
		
	global APP, PLUGIN
	
	settings = APP.settings
	dialog = plugins.PluginConfigDialog( APP.window, PLUGIN_NAME )
	box = dialog.main

	pauseVisible = settings.get_boolean( 'pause_visible', plugin=plugins.name(__file__), default = True )
	nextVisible = settings.get_boolean( 'next_visible', plugin=plugins.name(__file__), default = True )
	previousVisible = settings.get_boolean( 'previous_visible', plugin=plugins.name(__file__), default = True )
	showTrackInformation = settings.get_boolean( 'show_track_information', plugin=plugins.name(__file__), default = True )
	buttonReverse = settings.get_boolean( 'button_reverse',  plugin=plugins.name(__file__), default = False )

	pauseVisibleBox = gtk.CheckButton( _('Pause/Play icon') )
	nextVisibleBox = gtk.CheckButton( _('Next icon') )
	previousVisibleBox = gtk.CheckButton( _('Previous icon') )
	showTrackInformationBox = gtk.CheckButton( _('Show track information in the tooltip of Next and Previous') )
	buttonReverseBox = gtk.CheckButton( _('Reverse the order of the Next and Previous buttons (requires a restart of Exaile)') )

	pauseVisibleBox.set_active( pauseVisible )
	nextVisibleBox.set_active( nextVisible )
	previousVisibleBox.set_active( previousVisible )
	showTrackInformationBox.set_active( showTrackInformation )
	buttonReverseBox.set_active( buttonReverse )

	box.pack_start( pauseVisibleBox )
	box.pack_start( nextVisibleBox )
	box.pack_start( previousVisibleBox )
	box.pack_start( showTrackInformationBox )
	box.pack_start( buttonReverseBox )
	dialog.show_all()

	result = dialog.run()
	dialog.hide()

	settings.set_boolean( 'pause_visible', pauseVisibleBox.get_active(), plugin=plugins.name(__file__) )
	settings.set_boolean( 'next_visible', nextVisibleBox.get_active(), plugin=plugins.name(__file__) )
	settings.set_boolean( 'previous_visible', previousVisibleBox.get_active(), plugin=plugins.name(__file__) )
	settings.set_boolean( 'show_track_information', showTrackInformationBox.get_active(), plugin=plugins.name(__file__) )
	settings.set_boolean( 'button_reverse', buttonReverseBox.get_active(), plugin=plugins.name(__file__) )
	
	PLUGIN.setPauseVisible( pauseVisibleBox.get_active() )
	PLUGIN.setNextVisible( nextVisibleBox.get_active() )
	PLUGIN.setPreviousVisible( previousVisibleBox.get_active() )
	PLUGIN.setShowTrackInformation( showTrackInformationBox.get_active() )
	
def pause_toggled( exaile, track ):
	"""
		Called when pause is toggled
	"""
	
	global PLUGIN
	
	PLUGIN.setPauseIcon()
	
def play_track( exaile, track ):
	"""
		Called when playback on a track starts ("play-track" event)
	"""
	global PLUGIN
	global APP
	
	if ( PLUGIN.getShowTrackInformation() == True ) :
		if ( APP.tracks.get_next_track( track ) != None ) :
			PLUGIN.setNextTooltip( track )
		if ( APP.tracks.get_previous_track( track ) != None ) :
			PLUGIN.setPreviousTooltip( track )



#!/usr/bin/env python

# Copyright (C) 2007 Roman Koller
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#
# Title:				Exaile-cover-mover
# Description:		This program moves the covers fetched by Exaile directly to your music library.
# Requirements: 	- Exaile closed
# Known Bugs:		- Not able to handle files with non-Unicode characters
# Author: 			Roman Koller
# Date: 				11-09-2007
#

# imports
from pysqlite2 import dbapi2 as sqlite
import shutil, os.path, os

# the filename for covers
coverfile = "cover.jpg"

# let's go
exailepath = os.path.join(os.path.expanduser('~'), '.exaile')
dbfile = os.path.join(exailepath, 'music.db')
print "Opening database %s..." % dbfile,
exit

# open exaile database
db = sqlite.connect(dbfile)
cursor = db.cursor()

print "Done.\nGetting all albums with amazon cover...",
cursor.execute('SELECT a.image, p.name, a.id FROM tracks AS t, albums AS a, paths AS p WHERE t.path = p.id AND a.id = t.album AND a.image != \'\' AND a.image != \'nocover\' ORDER BY p.name') # LIMIT 200
print "Done.\nGetting paths...",

# put all paths and information to array paths
paths = []
for i in cursor:
	index = i[1].rfind(os.sep)
	# only once per path
	if [i[1][:index], i[0], i[2]] not in paths:
		paths.append([i[1][:index], i[0], i[2]])
		
print "Done.\nMoving %i covers..." % (len(paths))

# move picture to album folder
for i in paths:
	print "\t%s..." % i[0],
	try:
		shutil.move(os.path.join(exailepath, 'covers', str(i[1])), os.path.join(str(i[0]), coverfile))
	except IOError, (errno, strerror):
		if errno == 2:
			print "Database is not synchronised with your filesystem. There's already a cover. Skipping."
		else:
			print "IOError %i: %s" % (errno, strerror)
		continue
	except UnicodeEncodeError:
		print "Unfortunately, Python is not yet ready to deal with filenames that are not Unicode. Skipping."
		continue
	print "Done."
	
print "All covers moved."

again = True
while again:
	user_input = raw_input("Should all other covers be removed, too (and get lost) (y=yes*, n=no)? ").strip().lower()

	if not user_input or user_input == 'y':
		# update database, remove all amazon album covers
		print "Database update... ",
		try:
			cursor.execute('UPDATE albums SET image = \'\', amazon_image = 0') 
		except sqlite.OperationalError:
			print "Database is locked. Close Exaile and try again."
		print "Done.\nRemoving fetched but unused covers from filesystem...",
		
		# remove fetched but unused covers from filesystem
		shutil.rmtree(os.path.join(exailepath, 'covers'))
		os.mkdir(os.path.join(exailepath, 'covers'))
		print "Done."
		
		again = False
	
	elif user_input == 'n':
		print "Skipped.\nRemoving only moved covers from database..."
		for i in paths:
			try:
				cursor.execute("UPDATE albums SET image = '', amazon_image = 0 WHERE id = '%i'" %i[2])
			except sqlite.OperationalError:
				print "Database is locked. Close Exaile and try again."
		
		again = False


# submit db changes, close connection
db.commit()
cursor.close()	




# Copyright (C) 2006 Adam Olsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import thread, os, shlex, string, urllib2
from gettext import gettext as _
import pygtk, common
pygtk.require('2.0')
import gtk, gtk.glade
import cd_import, xlmisc, burn, advancededitor
import xl.path

#try:
#    import gamin
#    GAMIN_AVAIL = True
#except ImportError:
#    GAMIN_AVAIL = False
GAMIN_AVAIL = False

settings = None
xml = None
TEXT_VIEW_DEFAULT = """<big><b>{title}</b></big>
{artist}
on <i>{album}</i> - [{length}]"""

class PrefsItem(object):
    """
        Representing a gtk.Entry preferences item
    """
    def __init__(self, name, default, change=None, done=None):
        """
            Initializes the preferences item
            expects the name of the widget in the .glade file, the default for
            this setting, an optional function to be called when the value is
            changed, and an optional function to be called when this setting
            is applied
        """

        self.widget = xml.get_widget(('prefs_%s' % name).replace("/", "_"))
        self.name = name
        self.default = default
        self.change = change
        self.done = done

        self.set_pref()
        if change: 
            self.setup_change()

    def setup_change(self):
        """
            Sets up the function to be called when this preference is changed
        """
        self.widget.connect('focus-out-event',
            self.change, self.name, unicode(self.widget.get_text(), 'utf-8'))
        self.widget.connect('activate',
            lambda *e: self.change(self.widget, None, self.name,
                unicode(self.widget.get_text(), 'utf-8')))

    def set_pref(self):
        """ 
            Sets the GUI widget up for this preference
        """
        if not self.widget:
            xlmisc.log("Widget not found: %s" % (self.name))
            return
        self.widget.set_text(str(settings.get_str(self.name, default=self.default)))

    def do_done(self):
        """
            Calls the done function
        """
        return self.done(self.widget)

    def apply(self):
        """
            applies this setting
        """
        if self.done and not self.do_done(): return False
        settings.set_str(self.name, unicode(self.widget.get_text(), 'utf-8'))
        return True

class CryptedPrefsItem(PrefsItem):
    """
        An encrypted preferences item
    """
    def __init__(self, name, default, change=None, done=None):
        PrefsItem.__init__(self, name, default, change, done)

    def set_pref(self):
        self.widget.set_text(settings.get_crypted(self.name,
            default=self.default))

    def apply(self):
        if self.done and not self.do_done(): return False
        settings.set_crypted(self.name, unicode(self.widget.get_text(), 'utf-8'))
        return True

class PrefsTextViewItem(PrefsItem):
    """
        Represents a gtk.TextView
    """
    def __init__(self, name, default, change=None, done=None):
        """
            Initializes the object
        """
        PrefsItem.__init__(self, name, default, change, done)

    def setup_change(self):
        """
            Detects changes in this widget
        """
        self.widget.connect('focus-out-event',
            self.change, self.name, self.get_all_text())

    def get_all_text(self):
        """
            Returns the value of the text buffer
        """
        buf = self.widget.get_buffer()
        start = buf.get_start_iter()
        end = buf.get_end_iter()
        return unicode(buf.get_text(start, end), 'utf-8')

    def set_pref(self):
        """
            Sets the value of this widget
        """
        self.widget.get_buffer().set_text(str(settings.get_str(self.name,
            default=self.default)))

    def do_done(self):
        """
            Calls the done function
        """
        return self.done(self.widget)

    def apply(self):    
        """
            Applies the setting
        """
        if self.done and not self.do_done(): return False
        settings.set_str(self.name, self.get_all_text())
        return True
       
class CheckPrefsItem(PrefsItem):
    """
        A class to represent check boxes in the preferences window
    """
    def __init__(self, name, default, change=None, done=None):
        PrefsItem.__init__(self, name, default, change, done)

    def setup_change(self):
        self.widget.connect('toggled',
            self.change)

    def set_pref(self):
        self.widget.set_active(settings.get_boolean(self.name,
            self.default))

    def apply(self):
        if self.done and not self.do_done(): return False
        settings.set_boolean(self.name, self.widget.get_active())
        return True

class ListPrefsItem(PrefsItem):
    """
        A class to represent a space separated list in the preferences window
    """
    def __init__(self, name, default, change=None, done=None):
        PrefsItem.__init__(self, name, default, change, done)

    def set_pref(self):
        items = settings.get_list(self.name, default=self.default)
        try:
            items = " ".join(items)
        except:
            items = ""
        self.widget.set_text(items)

    def apply(self):
        if self.done and not self.do_done(): return False
        # shlex is broken with unicode, so we feed it UTF-8 and encode
        # afterwards.
        values = shlex.split(self.widget.get_text())
        values = [unicode(value, 'utf-8') for value in values]
        settings.set_list(self.name, values)
        return True

class SpinPrefsItem(PrefsItem):
    def set_pref(self):
        value = settings.get_float(self.name, default=self.default)
        self.widget.set_value(value)

class FloatPrefsItem(PrefsItem):
    """
        A class to represent a floating point number in the preferences window
    """
    def __init__(self, name, default, change=None, done=None):
        PrefsItem.__init__(self, name, default, change, done)

    def set_pref(self):
        self.widget.set_text(str(settings.get_float(self.name, default=self.default)))

    def apply(self):
        if self.done and not self.do_done(): return False
        settings.set_float(self.name, float(self.widget.get_text()))
        return True

class ColorButtonPrefsItem(PrefsItem):
    """
        A class to represent the color button in the prefs window
    """
    def __init__(self, name, default, change=None, done=None):
        PrefsItem.__init__(self, name, default, change, done)

    def setup_change(self):
        self.widget.connect('color-set',
            self.change, self.name)

    def set_pref(self):
        self.widget.set_color(gtk.gdk.color_parse(
            settings.get_str(self.name, self.default)))

    def apply(self):
        if self.done and not self.do_done(): return False
        color = self.widget.get_color()
        string = "#%.2x%.2x%.2x" % (color.red / 257, color.green / 257, 
            color.blue / 257)
        settings[self.name] = string
        return True

class FontButtonPrefsItem(ColorButtonPrefsItem):
    """
        Font button
    """
    def __init__(self, name, default, change=None, done=None):
        ColorButtonPrefsItem.__init__(self, name, default, change, done)

    def setup_change(self):
        self.widget.connect('font-set', self.change, self.name)

    def set_pref(self):
        font = settings.get_str(self.name, self.default)
        self.widget.set_font_name(font)
        
    def apply(self):
        if self.done and not self.do_don(): return False
        font = self.widget.get_font_name()
        settings[self.name] = font
        return True

class DirPrefsItem(PrefsItem):
    """
        Directory chooser button
    """
    def __init__(self, name, default, change=None, done=None):
        PrefsItem.__init__(self, name, default, change, done)

    def setup_change(self):
        pass

    def set_pref(self):
        """
            Sets the current directory
        """
        directory = os.path.expanduser(settings.get_str(self.name, self.default))
        if not os.path.exists(directory):
            os.makedirs(directory)
        self.widget.set_filename(directory)

    def apply(self):
        if self.done and not self.do_done(): return False
        directory = self.widget.get_filename()
        settings[self.name] = directory
        return True

class ComboPrefsItem(PrefsItem):
    """
        combo box
    """
    def __init__(self, name, default, change=None, done=None, 
        use_index=False):
        self.use_index = use_index
        PrefsItem.__init__(self, name, default, change, done)

    def setup_change(self):
        self.widget.connect('changed',
            self.change)

    def set_pref(self):
        item = settings.get_str(self.name, self.default)

        if self.use_index:
            index = settings.get_int(self.name, self.default)
            self.widget.set_active(index)
            return

        model = self.widget.get_model()
        iter = model.get_iter_first()
        count = 0
        while True:
            value = model.get_value(iter, 0)
            if value == item:
                self.widget.set_active(count)
                break
            count += 1
            iter = model.iter_next(iter)
            if not iter: break

    def apply(self):
        if self.done and not self.do_done(): return False

        if self.use_index:
            settings[self.name] = self.widget.get_active()
        else:
            settings[self.name] = self.widget.get_active_text()
        return True

class Preferences(object):
    """
        Preferences Dialog
    """

    CATEGORIES = [
        # TRANSLATORS: Category of the preferences dialog
        (_("General"), [
            # TRANSLATORS: Category of the preferences dialog
            _("Library"),
            # TRANSLATORS: Category of the preferences dialog
            _("Notification"),
            # TRANSLATORS: Category of the preferences dialog
            _("Importing"),
            # TRANSLATORS: Category of the preferences dialog
            _("Last.fm"),
            # TRANSLATORS: Category of the preferences dialog
            _("Radio"),
        ]),
        # TRANSLATORS: Category of the preferences dialog
        (_("Advanced"), [
            _("Replay Gain"),
            # TRANSLATORS: Category of the preferences dialog
            _("Proxy Settings")
        ]),
    ]

    def __init__(self, parent):
        """
            Initilizes the preferences dialog
        """

        global settings, xml

        self.exaile = parent
        settings = self.exaile.settings
        self.popup = None
        self.xml = gtk.glade.XML('exaile.glade', 'PreferencesDialog', 'exaile')
        xml = self.xml
        self.window = self.xml.get_widget('PreferencesDialog')
        self.window.set_transient_for(parent.window)
        self.window.connect('delete-event', lambda *e: self.cancel())
        self.xml.get_widget('show_advanced_button').connect('clicked',
            lambda *e: advancededitor.AdvancedConfigEditor(self.exaile,
            self.window, self, settings, 'data/settings_meta.ini'))

        self.nb = self.xml.get_widget('prefs_nb')
        self.nb.set_show_tabs(False)

        self.xml.get_widget('prefs_cancel_button').connect('clicked',
            lambda *e: self.cancel())
        self.xml.get_widget('prefs_apply_button').connect('clicked',
            self.apply)
        self.xml.get_widget('prefs_ok_button').connect('clicked',
            self.ok)

        self.label = self.xml.get_widget('prefs_frame_label')

        self.tree = self.xml.get_widget('prefs_tree')
        text = gtk.CellRendererText()
        col = gtk.TreeViewColumn(_('Preferences'), text, text=0)
        self.tree.append_column(col)

        self.model = gtk.TreeStore(str, int)
        self.tree.set_model(self.model)
        count = 0
        for cat, subcats in self.CATEGORIES:
            catnode = self.model.append(None, [cat, count])
            count += 1
            for subcat in subcats:
                self.model.append(catnode, [subcat, count])
                count += 1
            self.tree.expand_row(self.model.get_path(catnode), False)

        selection = self.tree.get_selection()
        selection.connect('changed', self.switch_pane)
        selection.select_path((0,))

        xml.get_widget('prefs_lastfm_pass').set_invisible_char('*')
        xml.get_widget('prefs_audio_sink').set_active(0)

        self.setup_settings()

    def setup_settings(self):
        """
            Sets the various widgets and settings preferences
        """
        global settings
        
        # populate the combobox with available burning programs
        burn_prog_combo = xml.get_widget('prefs_burn_prog')
        burn_progs = burn.check_burn_progs()
        if burn_progs:
            pref = settings.get_str('burn_prog', burn_progs[0])
            for i, prog in enumerate(burn_progs):
                burn_prog_combo.append_text(prog)
                if prog == pref:
                    burn_prog_combo.set_active(i)
        else:
            burn_prog_combo.append_text(_('No burning programs found'))
            burn_prog_combo.set_active(0)
            burn_prog_combo.set_sensitive(False)

        self.fields = []
        self.osd_settings = xlmisc.get_osd_settings(self.exaile.settings)
           
        # populate the import format combobox
        import_format = xml.get_widget('prefs_import_format')
        import_formats = cd_import.check_import_formats()
        if import_formats:
            pref = settings.get_str('import_format', import_formats[0])
            for i, format in enumerate(import_formats):
                import_format.append_text(format)
                if format == pref:
                    import_format.set_active(i)
        else:
            import_format.append_text(_('No suitable GStreamer plugins found'))
            import_format.set_active(0)
            import_format.set_sensitive(False)

        self.text_display = PrefsTextViewItem('osd/display_text',
            TEXT_VIEW_DEFAULT, self.display_popup)
        self.fields.append(self.text_display)
        self.fields.append(ComboPrefsItem('ui/tab_placement',
            0, None, self.setup_tabs, use_index=True))

        # trigger the toggle callback manually once to set up the right
        # sensitivity for the pref widgets in 'Importing'
        checkbox = xml.get_widget('prefs_import_use_custom')
        checkbox.connect('toggled', self.use_custom_toggled)
        self.use_custom_toggled(checkbox)

        location_button = xml.get_widget('prefs_import_location_button')
        location_entry = xml.get_widget('prefs_import_location')
        location_button.connect('clicked', self.choose_location, location_entry)

        # update the "example" label when values change
        naming = xml.get_widget('prefs_import_naming')
        naming.connect('changed', self.update_example)

        # update the "bitrate" label when quality gets changed
        quality = xml.get_widget('prefs_import_quality')
        bitrate = xml.get_widget('prefs_import_bitrate')
        quality.connect('changed', self.update_bitrate, bitrate)
        import_format.connect('changed', self.update_bitrate, bitrate)
        bitrate.set_no_show_all(True)
        bitrate.hide()
        
        # replaygain toggle handler
        replaygain = xml.get_widget('prefs_replaygain_enabled')
        replaygain.connect('toggled', self.toggle_replaygain)

        # proxy server handler
        proxy = xml.get_widget('prefs_proxy_enabled')
        proxy.connect('toggled', self.toggle_proxy)

        simple_settings = ({
            'ui/use_splash': (CheckPrefsItem, True),
#            'watch_directories': (CheckPrefsItem, False, self.check_gamin,
#                self.setup_gamin),
#            'watch_exclude_dirs': (ListPrefsItem, []),
            'fetch_covers': (CheckPrefsItem, True),
            'save_queue': (CheckPrefsItem, True),
            'ui/ensure_visible': (CheckPrefsItem, True),
            'art_filenames': (ListPrefsItem, 
                ['cover.jpg', 'folder.jpg', '.folder.jpg', 'album.jpg', 'art.jpg']),
            'scan_ignore': (ListPrefsItem, ['incomplete']),
            'open_last': (CheckPrefsItem, True),
            'osd/enabled': (CheckPrefsItem, True),
            'osd/w': (PrefsItem, '400', self.osd_adjust_size),
            'osd/h': (PrefsItem, '95', self.osd_adjust_size),
            'lastfm/user': (PrefsItem, ''),
            'lastfm/pass': (CryptedPrefsItem, '', None, self.setup_lastfm),
            'lastfm/submit': (CheckPrefsItem, True),
            'cd_device': (PrefsItem, '/dev/cdrom'),
            'audio_sink': (ComboPrefsItem, 'Use GConf Settings'), # FIXME: i18n?
            'burn_prog': (ComboPrefsItem, (burn_progs and burn_progs[0]) or ''),
            'osd/tray': (CheckPrefsItem, True),
            'osd/bgcolor': (ColorButtonPrefsItem, '#567ea2',
                self.osd_colorpicker),
            'osd/text_color': (ColorButtonPrefsItem, '#ffffff',
                self.osd_colorpicker),
            'osd/text_font': (FontButtonPrefsItem, 'Sans 10',
                self.osd_fontpicker),
            'osd/opacity': (SpinPrefsItem, 80, self.osd_adjust_opacity),
            'ui/use_tray': (CheckPrefsItem, False, None, self.setup_tray),
            'amazon_locale': (ComboPrefsItem, 'us'),
            'wikipedia_locale': (PrefsItem, 'en'),
            'scan_interval': (FloatPrefsItem, 25, None,
                self.setup_scan_interval),
            'feed_download_location': (DirPrefsItem, '~/.exaile/podcasts'),
            'import/format': (ComboPrefsItem, 
                (import_formats and import_formats[0]) or 'MP3'),
            'import/quality': (ComboPrefsItem, "High"), # FIXME: i18n?
            'import/location': (PrefsItem, '', None, self.import_location_changed),
            'import/naming': (PrefsItem, '${artist}/${album}/${artist} - ${title}.${ext}'),
            'import/use_custom': (CheckPrefsItem, False),
            'import/custom': (PrefsItem, ''),
            'replaygain/enabled': (CheckPrefsItem, True),
            'replaygain/album_mode': (CheckPrefsItem, True),
            'replaygain/preamp': (FloatPrefsItem, 0.0),
            'replaygain/fallback': (FloatPrefsItem, 0.0),
            'check_for_updates': (CheckPrefsItem, True),
            'proxy/enabled': (CheckPrefsItem, False, None, self.proxy_setting_changed),
            'proxy/server': (PrefsItem, '', None, self.proxy_setting_changed),
            'proxy/port': (PrefsItem, '', None, self.proxy_setting_changed),
            'proxy/username': (PrefsItem, '', None, self.proxy_setting_changed),
            'proxy/password': (CryptedPrefsItem, '', None, self.proxy_setting_changed),
            'ui/covers_in_collection': (CheckPrefsItem, False),
        })

        for setting, value in simple_settings.iteritems():
            c = value[0]
            default = value[1]
            if len(value) == 3: change = value[2]
            else: change = None

            if len(value) == 4: done = value[3]
            else: done = None
            item = c(setting, default, change, done)
            self.fields.append(item)

    setting_changed = setup_settings

    def advanced_toggle_cover(self, item, value):
        if value:
            self.exaile.xml.get_widget('main_cover_frame').show_all()
        else:
            self.exaile.xml.get_widget('main_cover_frame').hide()

    def advanced_toggle_tabbar(self, item, value):
        # value == always show
        self.exaile.playlists_nb.set_show_tabs(
            value or self.exaile.playlists_nb.get_n_pages() > 1)

    def advanced_alphabet_alert(self, item, value):
        """
            Shows an alert when the user changes the "show extended alphabet"
            setting
        """
        # TRANSLATORS: Notification about the "Show Extended Alphabet" setting
        common.info(self.exaile.window, _('Click the "Refresh" button in '
            'the Collection panel to apply this setting.'))

    def advanced_toggle_panel(self, item, value):
        item = item.name.replace('ui/show_', '').replace('_panel', '')
        self.exaile.set_panel_visible(item, value)

    def advanced_toggle_clear_button(self, item, value):
        """
            Hides or shows the clear button
        """
        if value:
            self.exaile.clear_button.show()
        else:
            self.exaile.clear_button.hide()

    def advanced_toggle_stop_button(self, item, value):
        """
            Hides or shows the stop button
        """
        if value:
            self.exaile.stop_button.show()
        else:
            self.exaile.stop_button.hide()        

    def setup_scan_interval(self, widget):
        """
            Makes sure the scan interval is valid, and starts it over with the
            new time value
        """
        value = widget.get_text()

        try:    
            value = float(value)
        except ValueError:  
            common.error(self.exaile.window, "%s is an invalid value "
                "for the library rescan interval" % value)
            return False

        self.exaile.start_scan_interval(value)
        return True

    def update_bitrate(self, widget, bitrate_label):
        """
            Update the bitrate label
        """
        quality = self.xml.get_widget('prefs_import_quality').get_active_text()
        if not quality: return
        quality_label = self.xml.get_widget('quality_label')
        format = self.xml.get_widget('prefs_import_format').get_active_text()

        if format == "FLAC":
            quality_label.set_text(_('Compression Level:'))
            bitrate_label.hide()
        else:
            quality_label.set_text(_('Quality:'))
            bitrate = cd_import.formatdict[format][quality]
            if format == "MP3":
                # TRANSLATORS: For MP3 CBR
                bitrate_label.set_text(_("%d kbps") % bitrate)
            elif format == "MP3 VBR":
                # TRANSLATORS: For MP3 VBR
                bitrate_label.set_text(_("%d mean kbps") % bitrate)
            elif format == "Ogg Vorbis":
                bitrate_label.set_text(str(bitrate))
            bitrate_label.show()


    def proxy_setting_changed(self, widget):
        """
            Apply proxy settings
        """
        if self.exaile.settings.get_boolean('proxy/enabled', False):
            server = self.exaile.settings.get_str('proxy/server', "")
            port = self.exaile.settings.get_str('proxy/port', "")
            username = self.exaile.settings.get_str('proxy/username', "")
            password = self.exaile.settings.get_str('proxy/password', "")

            proxy_url = "http://"

            if username != "" and password != "":
                proxy_url += "%s:%s@" % (username, password)

            proxy_url += server

            if port != "":
                proxy_url += ":%s" % port

            proxy_support = urllib2.ProxyHandler({'http':proxy_url})
            opener = urllib2.build_opener(proxy_support, urllib2.HTTPHandler)
            urllib2.install_opener(opener)
        else:
            opener = urllib2.build_opener(urllib2.HTTPHandler)
            urllib2.install_opener(opener)

        return True

    def import_location_changed(self, widget):
        """
            Scan the new import location for new songs
        """
        items = self.exaile.settings.get_list('search_paths', '')
        path = unicode(widget.get_text(), 'utf-8')
        if not path: return True

        new_items = []
        for item in items:
            if path.startswith(item): 
                # if the import location is a subdir of some existing library path
                # don't scan the tracks since they already are in the library
                self.exaile.settings['import/scan_import_dir'] = False
                return True
            if not item.startswith(path):
                # if the import location is a parent of some existing library path
                # consolidate the list (by not including the old path anymore)
                new_items.append(item)

        self.exaile.settings['search_paths'] = new_items
        self.exaile.library_manager.update_library_add([path], load_tree=True)
        return True

    def choose_location(self, widget, location_entry):
        """
            Sets the "location" preference
        """
        dialog = gtk.FileChooserDialog(_("Choose a directory"),
            self.exaile.window, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
            gtk.STOCK_OK, gtk.RESPONSE_OK))
        response = dialog.run()
        dialog.hide()
        
        if response != gtk.RESPONSE_OK:
            return

        path = dialog.get_filename()

        location_entry.set_text(path)

    def check_gamin(self, widget):
        """
            Make sure gamin is availabe
        """
        if widget.get_active():
            if not GAMIN_AVAIL:
                common.error(self.exaile.window,
                    _("Cannot watch directories for changes. "
                    "Install python2.4-gamin to use this feature."))
                widget.set_active(False)
                return False

    def toggle_replaygain(self, widget, event=None):
        """
            Enables/disables replaygain options.
        """
        items = [xml.get_widget('prefs_replaygain_album_mode'),
                 xml.get_widget('prefs_replaygain_preamp'),
                 xml.get_widget('prefs_replaygain_fallback')]
        to_state = widget.get_active()

        for item in items:
            item.set_sensitive(to_state)

    def toggle_proxy(self, widget, event=None):
        """
            Enables/disables proxy server options.
        """
        item = xml.get_widget('prefs_proxy_table')
        to_state = widget.get_active()
        item.set_sensitive(to_state)

    def use_custom_toggled(self, widget, event=None):
        """
            Set sensitivity of widgets when checkbox is toggled
        """
        active = widget.get_active()

        #for w in ['format', 'quality', 'location', 'naming', 'example']:
        for w in ['format', 'quality']:
            xml.get_widget('prefs_import_' + w).set_sensitive(not active)
        xml.get_widget('prefs_import_custom').set_sensitive(active)
        return True

    def update_example(self, widget, *args):
        """
            Updates the 'example' label in the import prefs
        """
        example = xml.get_widget('prefs_import_example')
        #location = xml.get_widget('prefs_import_location')
        #naming = xml.get_widget('prefs_import_naming')

        #dir = location.get_filename()
        path = unicode(widget.get_text(), 'utf-8')

        # TODO: get proper mapping
        mapping = {'artist':'Artist', 'album':'Album', 'num':1, 
                    'title':'Title', 'ext':'mp3', 'date':'2000'}
        template = string.Template(path)
        path = template.safe_substitute(mapping)
        
        # this fails a few times because for some reason dir is often empty 
        # when the pref dialog is constructed... we ignore it
        try:
            #example.set_text(os.path.join(dir, path))
            example.set_text(path)
        except:
            pass

    def setup_lastfm(self, widget):
        """
            Connects to last.fm if the password field isn't empty
        """
        # The audioscrobbler module can handle unicode or UTF-8 str;
        # no need to decode.
        password = widget.get_text()
        if not password: return True
        user = xml.get_widget('prefs_lastfm_user').get_text()

        self.exaile.scrobbler_login(user, password)
        return True

    def setup_gamin(self, widget):
        """
            Sets up gamin if needs be
        """
        if widget.get_active() and not self.exaile.mon:
            self.exaile.setup_gamin(True)
        return True

    def setup_tabs(self, widget, *p):
        """
            Sets up tab placement for the playlists tab
        """

        index = widget.get_active()
        self.exaile.set_tab_placement(index)
        return True

    def ok(self, widget):
        """
            Called when the user clicks 'ok'
        """
        if self.apply(None): 
            self.cancel()
            self.window.hide()
            self.window.destroy()

    def apply(self, widget):
        """
            Applies settings
        """
        for item in self.fields:
            if not item.apply():
                print item.name
                return False

        xlmisc.POPUP = None

        return True

    def setup_tray(self, widget, event=None):
        """
            Sets up the tray icon
        """
        val = widget.get_active()

        if val: self.exaile.setup_tray()
        else: self.exaile.remove_tray()
        return True

    def advanced_setup_tray(self, item, value):
        if value: self.exaile.setup_tray()
        else: self.exaile.remove_tray()

    def osd_adjust_size(self, widget, event, name, previous):
        """
            Called when the user requests to adjust the size of the osd
        """

        try:
            val = int(widget.get_text())
        except ValueError:
            widget.set_text(previous)
            return
        
        self.osd_settings[name] = val
        self.display_popup()

    def osd_adjust_opacity(self, widget, event, name, previous):
        """
            Called when the user requests to adjust the opacity (alpha value) of the osd
        """
        try:
            val = int(widget.get_text())
        except ValueError:
            widget.set_text(previous)
            return

        self.osd_settings[name] = val
        self.display_popup()

    def osd_fontpicker(self, widget, name):
        """
            Gets the font from the font picker, and re-sets up the OSD window
        """

        self.osd_settings[name] = widget.get_font_name()
        self.display_popup()

    def osd_colorpicker(self, widget, name):
        """
            Shows a colorpicker
        """
        color = widget.get_color()
        string = "#%.2x%.2x%.2x" % (color.red / 257, color.green / 257, 
            color.blue / 257)

        self.osd_settings[name] = string
        self.display_popup()

    def cancel(self):
        """
            Closes the preferences dialog, ensuring that the osd popup isn't
            still showing
        """
        if self.popup:
            self.popup.window.hide()
            self.popup.window.destroy()
        self.window.hide()
        self.window.destroy()

    def switch_pane(self, selection):
        """
            Switches a pane
        """
        (model, iter) = selection.get_selected()
        if not iter: return
        index = self.model.get_value(iter, 1)
        self.nb.set_current_page(index)
        page = self.nb.get_nth_page(index)
        title = self.nb.get_tab_label(page)
        self.label.set_markup("<b>%s</b>" %
            common.escape_xml(title.get_label()))
        if index == 2: 
            self.osd_settings = xlmisc.get_osd_settings(self.exaile.settings)
            self.display_popup()
        else:
            if self.popup:
                self.popup.window.destroy()
                self.popup = None

    def display_popup(self, *e):
        """
            Shows the OSD window
        """
        if self.popup:  
            (x, y) = self.popup.window.get_position()
            self.osd_settings['osd/x'] = x
            self.osd_settings['osd/y'] = y
            self.popup.window.destroy()

        self.popup = xlmisc.OSDWindow(self.exaile, self.osd_settings,
            False, True)

        track = BlankClass() 
        for item in ('title', 'artist', 'album', 'length', 'track', 'bitrate',
            'genre', 'year', 'rating'):
            setattr(track, item, item)

        display_text = self.text_display.get_all_text()
        display_text = display_text.replace("{volume}", "\\{volume\\}")

        self.popup.show_track_osd(track, display_text, 
            xl.path.get_data('images', 'nocover.png'))

        return False

    def run(self):
        """
            Runs the dialog
        """
        self.window.show_all()

class BlankClass(object):
    pass



#!/usr/bin/env python

# Copyright (C) 2006 Adam Olsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import sys, os, urllib, re
from gettext import gettext as _
import pygtk
pygtk.require('2.0')
import gtk, gtk.glade, gobject
import xl.plugins
import xl.path, locale
from xl import common, xlmisc

def show_error(parent, message):
    """
        Shows an error dialog
    """
    dialog = gtk.MessageDialog(parent, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
        gtk.BUTTONS_OK, message)
    dialog.run()
    dialog.destroy()

class PluginManager(object):
    """
        Gui to manage plugins
    """
    def __init__(self, app, parent, manager, update, avail_url=''):
        """
            Initializes the manager
            params: parent window, plugin manager
        """
        self.app = app
        self.parent = parent
        self.update = update
        self.manager = manager
        self.lists = dict()
        self.models = dict()
        self.author_labels = dict()
        self.descriptions = dict()
        self.version_labels = dict()
        self.name_labels = dict()

        self.xml = gtk.glade.XML('xl/plugins/plugins.glade',
            'PluginManagerDialog','exaile')
        self.dialog = self.xml.get_widget('PluginManagerDialog')
        self.dialog.set_transient_for(parent)
        self.plugin_nb = self.xml.get_widget('plugin_notebook')

        self.list = self.xml.get_widget('plugin_tree')
        self.version_label = self.xml.get_widget('version_label')
        self.author_label = self.xml.get_widget('author_label')
        self.name_label = self.xml.get_widget('name_label')
        self.description = self.xml.get_widget('description_view')
        self.model = gtk.ListStore(gtk.gdk.Pixbuf, str, bool, object)
        self.xml.get_widget('close_button').connect('clicked',
            lambda *e: self.dialog.destroy())
        self.configure_button = self.xml.get_widget('configure_button')
        self.configure_button.connect('clicked', self.configure_plugin)
        self.plugin_install_button = self.xml.get_widget('plugin_install_button')
        self.plugin_install_button.connect('clicked',
            lambda *e: self.install_plugin('avail'))
        self.plugin_update_button = self.xml.get_widget('plugin_update_button')
        self.plugin_update_button.connect('clicked',
            lambda *e: self.install_plugin('update'))
        self.xml.get_widget('plugin_uninstall_button').connect('clicked',
            self.uninstall_plugin)

        pb = gtk.CellRendererPixbuf()
        text = gtk.CellRendererText()
        # TRANSLATORS: The column title for plugin names
        col = gtk.TreeViewColumn(_("Plugin"))
        col.pack_start(pb, False)
        col.pack_start(text, False)
        col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
        col.set_fixed_width(1)
        col.set_expand(True)
        col.set_attributes(pb, pixbuf=0)
        col.set_attributes(text, text=1)

        self.list.append_column(col)

        text = gtk.CellRendererToggle()
        text.set_property('activatable', True)
        text.connect('toggled', self.toggle_cb, self.model)
        # TRANSLATORS: The column title for the plugin statuses
        col = gtk.TreeViewColumn(_("Enabled"), text)
        col.add_attribute(text, 'active', 2)
        col.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        self.list.append_column(col)

        self.load_plugin_list()

        selection = self.list.get_selection()
        selection.connect('changed', self.row_selected)
        self.list.set_model(self.model)
        self.dialog.show_all()
        selection.select_path(0)
        self.fetched = False

        if avail_url:
            self.setup_tab('avail')
            self.setup_tab('update')
            self.avail_url = avail_url
            self.plugin_nb.connect('switch-page', lambda *e: self.check_fetch('avail'))

    def load_plugin_list(self):
        self.model.clear()
        list = []
        for plugin in self.manager.plugins:
            icon = plugin.PLUGIN_ICON
            if not icon:
                icon = self.dialog.render_icon('gtk-execute',
                    gtk.ICON_SIZE_MENU)

            list.append([icon, plugin.PLUGIN_NAME, plugin.PLUGIN_ENABLED, 
                plugin])

        def plugin_sort(a, b):
            return locale.strcoll(a[1].lower(), b[1].lower())

        list.sort(plugin_sort)
        for p in list:
            self.model.append(p)

    def install_plugin(self, type='avail'):
        """
            Installs the selected plugin
        """
        self.plugin_install_button.set_sensitive(False)
        self.plugin_update_button.set_sensitive(False)
        self.plugin_nb.set_sensitive(False)
        self.download_plugins(type)

    def done_installing(self, files, type='avail'):
        # now we remove all the installed plugins from the available list
        while True:
            iter = self.models[type].get_iter_first()
            while True:
                if not iter: break
                file = self.models[type].get_value(iter, 4)
                if file in files:
                    self.models[type].remove(iter)
                    break
                iter = self.models[type].iter_next(iter)
            if not iter: break
        self.plugin_install_button.set_sensitive(True)
        self.plugin_update_button.set_sensitive(True)
        self.load_plugin_list()
        self.plugin_nb.set_sensitive(True)

        self.version_labels[type].set_text('')
        self.author_labels[type].set_text('')
        self.name_labels[type].set_markup('<b>' + _('No Plugin Selected') +
            '</b>')
        self.descriptions[type].get_buffer().set_text('')

    @common.threaded
    def download_plugins(self, type='avail'):
        """
            Downloads the selected plugin
        """
        download_dir = xl.path.get_config('plugins')
        files = []
        iter = self.models[type].get_iter_first()
        while True:
            if not iter: break

            checked = self.models[type].get_value(iter, 5)
            if checked:
                file = self.models[type].get_value(iter, 4)

                try:
                    # if the directory does not exist, create it
                    if not os.path.isdir(download_dir):
                        os.mkdir(download_dir, 0777)
                    download_url = "http://www.exaile.org/files/plugins/%s/%s" \
                        % (self.app.get_plugin_location(), file)
                    xlmisc.log('Downloading %s from %s' % (file, download_url))

                    plugin = urllib.urlopen(download_url).read()
                    h = open(os.path.join(download_dir, file), 'w')
                    h.write(plugin)
                    h.close()

                    try:
                        _name = re.sub(r'\.py[co]?$', '', file)
                        if _name in sys.modules:
                            del sys.modules[_name]
                    except Exception, e:
                        xlmisc.log_exception()

                    enabled_plugins = []
                    avail_plugins = []
                    for k, v in self.app.settings.get_plugins().iteritems():
                        if v:
                            if k.endswith(".exz"):
                                enabled_plugins.append(k.replace('.exz', '.py'))
                            else:
                                enabled_plugins.append("%s.py" % k)
                        else:
                            if k.endswith(".exz"):
                                avail_plugins.append(k.replace('.exz', '.py'))
                            else:
                                avail_plugins.append("%s.py" % k)

                    gobject.idle_add(self.manager.initialize_plugin, download_dir, file, 
                        enabled_plugins, False, avail_plugins)
                    files.append(file)
                except Exception, e:
                    self.models[type].set_value(iter, 5, False)
                    gobject.idle_add(common.error, self.parent, _("%(plugin)s could "
                        "not be installed: %(exception)s") % 
                        {
                            'plugin': file, 
                            'exception': e
                        })
                    xlmisc.log_exception()
            iter = self.models[type].iter_next(iter)

        gobject.idle_add(self.done_installing, files, type)

    def check_fetch(self, type='avail'):
        """
            Checks to see if the available plugin list needs to be fetched
        """
        if self.plugin_nb.get_current_page() != 0 or self.fetched: return
        self.fetched = True
        self.version_labels[type].set_text('')
        self.author_labels[type].set_text('')
        self.name_labels[type].set_markup('<b>' + _('No Plugin Selected') + '</b>')
        self.descriptions[type].get_buffer().set_text(_("Fetching available"
            " plugin list..."))
        self.models[type].clear()
        self.fetch_available_plugins(self.avail_url)
        xlmisc.log('Fetching available plugin list')

    @common.threaded
    def fetch_available_plugins(self, url):
        """
            Fetches a plugin list from the specified url
        """
        h = urllib.urlopen(url)
        lines = h.readlines()
        h.close()
        gobject.idle_add(self.done_fetching, lines)

    def done_fetching(self, lines):

        plugin_list = []
        update_list = []
        check = False
        for line in lines:
            line = line.strip()
            (file, name, version, author, description) = line.split('\t')
            description = description.replace("\n", " ").replace(r'\n', '\n')

            icon = self.dialog.render_icon('gtk-execute',
                gtk.ICON_SIZE_MENU)

            found = False
            for plugin in self.manager.plugins:
                if plugin.PLUGIN_NAME == name:

                    #this is a bit odd, to allow non-decimal versioning.
                    installed_ver = map(int, plugin.PLUGIN_VERSION.split('.'))
                    available_ver = map(int, version.split('.'))
            
                    found = True

                    # if there is an update available
                    if installed_ver < available_ver:
                        update_list.append([name, version, author,
                            description, file, False])
                        check = True

            if not found:
                plugin_list.append([name, version, author, description, file, False])
                check = True

        if not check:
            common.info(self.parent, _("No plugins or updates could be found "
                "for your version."))
        else:
            def plugin_sort(a, b):
                return locale.strcoll(a[0].lower(), b[0].lower())

            plugin_list.sort(plugin_sort)
            update_list.sort(plugin_sort)
            for plugin in plugin_list:
                self.models['avail'].append(plugin)
            for plugin in update_list:
                self.models['update'].append(plugin)

        for type in self.descriptions.keys():
            self.descriptions[type].get_buffer().set_text(_("No plugin selected"))
            selection = self.lists[type].get_selection()
            selection.select_path(0)
            self.t_row_selected(type, selection)

    def setup_tab(self, type='avail'):
        """
            Sets up the "plugins available" tab
        """
        self.models[type] = gtk.ListStore(str, str, str, str,
            str, bool)
        self.lists[type] = self.xml.get_widget('%s_plugin_tree' % type)
        self.version_labels[type] = self.xml.get_widget('%s_version_label' %
            type)
        self.author_labels[type] = self.xml.get_widget('%s_author_label' %
            type)
        self.name_labels[type] = self.xml.get_widget('%s_name_label' % type)
        self.descriptions[type] = self.xml.get_widget('%s_description_view' %
            type)

        text = gtk.CellRendererText()
        col = gtk.TreeViewColumn(_('Plugin'))
        col.pack_start(text, False)
        col.set_expand(True)
        col.set_fixed_width(120)
        col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
        col.set_attributes(text, text=0)
        self.lists[type].append_column(col)

        text = gtk.CellRendererText()
        # TRANSLATORS: The column title for plugin versions
        col = gtk.TreeViewColumn(_('Ver'))
        col.pack_start(text, False)
        col.set_expand(False)
        col.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        col.set_attributes(text, text=1)
        self.lists[type].append_column(col)

        text = gtk.CellRendererToggle()
        text.set_property('activatable', True)
        text.connect('toggled', self.t_toggle_cb, self.models[type])
        # TRANSLATORS: The column title for the plugin installation statuses
        col = gtk.TreeViewColumn(_("Inst"), text)
        col.add_attribute(text, 'active', 5)
        col.set_expand(False)
        col.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        self.lists[type].append_column(col)

        self.lists[type].set_model(self.models[type])
        self.descriptions[type].get_buffer().set_text(_("Fetching available"
            " plugin list..."))
        selection = self.lists[type].get_selection()
        selection.connect('changed', lambda selection, type=type: 
            self.t_row_selected(type, selection))

    def t_row_selected(self, type, selection):
        """
            Called when a user selects a row in the avialable tab
        """
        model, iter = selection.get_selected()
        if not iter: return

        name = model.get_value(iter, 0)
        version = model.get_value(iter, 1)
        author = model.get_value(iter, 2)
        description = model.get_value(iter, 3)

        self.name_labels[type].set_markup('<b>%s</b>' % common.escape_xml(name))
        self.version_labels[type].set_label(version)
        self.author_labels[type].set_label(author)
        self.descriptions[type].get_buffer().set_text(description)

    def t_toggle_cb(self, cell, path, model):
        model[path][5] = not model[path][5]

    def toggle_cb(self, cell, path, model):
        """
            Called when the checkbox is toggled
        """
        plugin = model[path][3]
        model[path][2] = not model[path][2]
        plugin.PLUGIN_ENABLED = model[path][2]
        if plugin.PLUGIN_ENABLED:
            try:
                if not plugin.initialize():
                    plugin.PLUGIN_ENABLED = False
                    print "Error initializing plugin"
                    model[path][2] = False
            except xl.plugins.PluginInitException, e:
                show_error(self.parent, str(e))
                model[path][2] = False
        else:
            plugin.destroy()
            plugin.PLUGIN_ENABLED = False

        xlmisc.log("Plugin %s set to enabled: %s" % (plugin.PLUGIN_NAME,
            plugin.PLUGIN_ENABLED))
        if self.update:
            self.update(plugin)

    def uninstall_plugin(self, *e):
        """
            Disables and uninstalls the selected plugin
        """
        result = common.yes_no_dialog(self.parent, _("Are you sure "
            "you want to uninstall the selected plugin?"))
        if result == gtk.RESPONSE_YES:
            selection = self.list.get_selection()
            (model, iter) = selection.get_selected()
            if not iter: return
            plugin = model.get_value(iter, 3)
            if plugin.PLUGIN_ENABLED:
                plugin.destroy()
            model.remove(iter)
            self.manager.plugins.remove(plugin)
            self.manager.loaded.remove(plugin.PLUGIN_NAME)

            try:
                del sys.modules[re.sub(r'\.py[co]?$', '', plugin.FILE_NAME)]

                filename = xl.path.get_config('plugins', plugin.FILE_NAME)
                if hasattr(plugin, '_IS_EXZ') and plugin._IS_EXZ:
                    # if it's an exz, remove that file instead of trying to remove
                    # the python files themselves
                    os.remove(filename.replace('.py', '.exz'))
                else:
                    os.remove(filename)
                    if os.path.isfile(filename + 'c'):
                        os.remove(filename + 'c') # pyc
                    if os.path.isfile(filename + 'o'):
                        os.remove(filename + 'o') # pyo
            except:
                xlmisc.log_exception()
            self.fetched = False

    def configure_plugin(self, *e):
        """
            Calls the plugin's configure function
        """
        selection = self.list.get_selection()
        (model, iter) = selection.get_selected()
        if not iter: return

        plugin = model.get_value(iter, 3)
        plugin.configure()

    def row_selected(self, selection, user_data=None):
        """
            Called when a row is selected
        """
        (model, iter) = selection.get_selected()
        if not iter: return

        plugin = model.get_value(iter, 3)
        self.version_label.set_label(str(plugin.PLUGIN_VERSION))
        self.author_label.set_label(", ".join(plugin.PLUGIN_AUTHORS))
        self.description.get_buffer().set_text(plugin.PLUGIN_DESCRIPTION.replace(
            "\n", " ").replace(r'\n', "\n"))
        self.name_label.set_markup("<b>%s</b>" %
            common.escape_xml(plugin.PLUGIN_NAME))
        self.configure_button.set_sensitive(hasattr(plugin, 'configure'))

class PluginTest(object):
    def __init__(self, name, version, author, desc):
        self.PLUGIN_NAME = name
        self.PLUGIN_VERSION = version
        self.PLUGIN_AUTHORS = [author]
        self.PLUGIN_DESCRIPTION = desc
        self.PLUGIN_ICON = None
        self.PLUGIN_ENABLED = True



#!/usr/bin/env python

import re, os, cgi, time, urllib, sys
os.chdir('/var/www/exaile.org/htdocs/plugins')
import os.path
from mod_python import apache
from mod_python.util import FieldStorage

def get_plugin(info, req):
    version = info['version'].value
    plugin = info['plugin'].value
    req.write(open('plugincache/%s/%s' % (version, plugin)).read())
    return apache.OK

def handler(req):
    info = FieldStorage(req)

    req.content_type = 'Content-type: text/plain'
    if info.has_key('plugin'):
        return get_plugin(info, req)

    name_re = re.compile(r'PLUGIN_NAME\s+=\s+r?(\'\'?\'?|""?"?)(.*?)(\1)', re.DOTALL|re.MULTILINE)
    version_re = re.compile(r'PLUGIN_VERSION\s+=\s+r?(\'\'?\'?|""?"?)(.*?)(\1)', re.DOTALL|re.MULTILINE)
    author_re = re.compile(r'PLUGIN_AUTHORS\s+=\s(\[.*?\])', re.DOTALL|re.MULTILINE)
    description_re = re.compile(r'PLUGIN_DESCRIPTION\s+=\s+r?(\'\'?\'?|""?"?)(.*?)(\1)', re.DOTALL|re.MULTILINE)
    plugin_re = re.compile(r'<a class="file" title="View File" href=".*?">(\w+.py)</a>', re.DOTALL|re.MULTILINE)

    h = open('new_pluginlog.txt', 'a+')
    date = time.strftime("[%Y-%m-%d %H:%M:%S]", time.localtime())
    h.write("%s\t%s\t%s\n" % (date, os.getenv('REMOTE_ADDR'),
        info['version'].value))
    h.close()

    version = info['version'].value
    data = urllib.urlopen('http://exaile.org/trac/browser/plugins/%s' %
        version).read()

    try:
        h = open('plugins_%s.cache' % info['version'].value, 'r')
        check_size = h.readline().strip()
        lines = h.readlines()
        h.close()
        if str(len(data)) == check_size:
            for line in lines:
                req.write(line)
            return apache.OK
    except IOError:
        # plugin cache doesn't exist, move on
        pass

    h = open('plugins_%s.cache' % info['version'].value, 'w')
    h.write("%d\n" % len(data))
    plugins = plugin_re.findall(data)

    appversion = version
    if not os.path.isdir('plugincache/%s' % version):
        os.mkdir('plugincache/%s' % version, 0777) 

    for plugin in plugins:
        data = \
            urllib.urlopen('http://www.exaile.org/trac/browser/plugins/%s/%s?format=txt'
            % (info['version'].value, plugin)).read()

        ph = open('plugincache/%s/%s' % (appversion, plugin), 'w+')
        ph.write(data)
        ph.close()

        m = name_re.search(data)
        if not m: continue

        name = m.group(2)

        m = version_re.search(data)
        if not m: continue

        version = m.group(2)

        m = author_re.search(data)
        if not m: continue

        author = eval(m.group(1))
        author = ", ".join(author)

        m = description_re.search(data)
        if not m: continue

        description = m.group(2)
        description = description.replace('\n', ' ')

        text = "%s\t%s\t%s\t%s\t%s" % (plugin, name, version, author,
            description)
        req.write(text + "\n")
        h.write("%s\n" % text)

    h.close()
    return apache.OK



#!/usr/bin/env python
# Copyright (C) 2006 Adam Olsen <arolsen@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import gtk, gobject, pango, gst
from gettext import gettext as _
from xl import xlmisc
import xl.plugins as plugins

PLUGIN_NAME = _("Mini Mode")
PLUGIN_AUTHORS = ['Adam Olsen <arolsen@gmail.com>']
PLUGIN_VERSION = '0.4.9'
PLUGIN_DESCRIPTION = _(r"""Super groovy mini mode window!\n\nMini Mode is activated
by pressing CTRL+ALT+M\n\nYou can move the window in most cases by
ALT+drag""")
PLUGIN_ENABLED = False
PLUGIN_ICON = None

PLUGIN = None
MENU_ITEM = None
ACCEL_GROUP = None
MM_ACTIVE = False
BUTTON = None

CONS = plugins.SignalContainer()

def toggle_minimode(*e):
    """
        Toggles Minimode
    """
    global MM_ACTIVE 
    if not PLUGIN.get_property("visible"):
        PLUGIN.show_window()
        APP.window.hide()
        MM_ACTIVE = True
    else:
        PLUGIN.hide()
        MM_ACTIVE = False
        APP.window.show()
    print "Minimode toggled"

def configure():
    """
        Configuration for mini mode
    """
    exaile = APP
    settings = exaile.settings

    dialog = plugins.PluginConfigDialog(exaile.window, PLUGIN_NAME)
    box = dialog.main

    on_top = settings.get_boolean('on_top', plugin=plugins.name(__file__), default=False)
    taskbar = settings.get_boolean('taskbar',
        plugin=plugins.name(__file__), default=True)
    decoration = settings.get_boolean('decoration',
        plugin=plugins.name(__file__), default=False)
    switchbutton = settings.get_boolean('switchbutton',
        plugin=plugins.name(__file__), default=False)
    font_size = settings.get_int('font_size', plugin=plugins.name(__file__), default=8)

    on_top_box = gtk.CheckButton(_('Always on top'))
    taskbar_box = gtk.CheckButton(_('Show in taskbar'))
    decoration_box = gtk.CheckButton(_('Window decoration'))
    switchbutton_box = gtk.CheckButton(_('Mini Mode button in main window'))

    font_size_label = gtk.Label(_('Tracklist font size:'))
    adjustment = gtk.Adjustment(value=font_size, lower=1, upper=72, step_incr=1)
    font_size_spinner = gtk.SpinButton(adjustment)
    font_size_spinner.set_numeric(True)

    font_size_box = gtk.HBox(spacing=3)
    font_size_box.pack_start(font_size_label)
    font_size_box.pack_start(font_size_spinner)

    on_top_box.set_active(on_top)
    taskbar_box.set_active(taskbar)
    decoration_box.set_active(decoration)
    switchbutton_box.set_active(switchbutton)

    box.pack_start(on_top_box)
    box.pack_start(taskbar_box)
    box.pack_start(decoration_box)
    box.pack_start(switchbutton_box)
    box.pack_start(font_size_box)
    dialog.show_all()

    result = dialog.run()
    dialog.hide()
    if not result == gtk.RESPONSE_OK: return

    if switchbutton_box.get_active():
        BUTTON.show()
    else:
        BUTTON.hide()

    settings.set_boolean('on_top', on_top_box.get_active(), plugin=plugins.name(__file__))
    settings.set_boolean('taskbar', taskbar_box.get_active(), plugin=plugins.name(__file__))
    settings.set_boolean('decoration', decoration_box.get_active(), plugin=plugins.name(__file__))
    settings.set_int('font_size', font_size_spinner.get_value_as_int(), plugin=plugins.name(__file__))
    settings.set_boolean('switchbutton', switchbutton_box.get_active(), plugin=plugins.name(__file__))

class MiniWindow(gtk.Window):
    """
        The main minimode window
    """
    def __init__(self):
        """
            Initializes the minimode window
        """
        gtk.Window.__init__(self)

        self.set_title("Exaile")
        self.set_icon(APP.window.get_icon())
        self.tips = gtk.Tooltips()
        self.seek_id = None
        self.seeking = False

        main = gtk.VBox()
        main.set_border_width(3)
        self.tlabel = gtk.Label("")
        self.tlabel.set_size_request(120, 10)
        self.tlabel.set_alignment(0.0, 0.5)

        bbox = gtk.HBox()
        bbox.set_spacing(3)

        prev = self.create_button('gtk-media-previous', self.on_prev,
            _('Previous'))
        bbox.pack_start(prev, False)

        self.play = self.create_button('gtk-media-play', self.on_play,
            _('Play/Pause'))
        bbox.pack_start(self.play, False)

        self.next = self.create_button('gtk-media-next', self.on_next, _('Next'))
        bbox.pack_start(self.next)

        self.model = gtk.ListStore(str, object)
        self.title_box = gtk.ComboBox(self.model)
        cell = gtk.CellRendererText()
        font_size = APP.settings.get_int('font_size', plugin=plugins.name(__file__), default=8)
        cell.set_property('font-desc', pango.FontDescription('Normal %d' % font_size))
        cell.set_property('ellipsize', pango.ELLIPSIZE_END)
    
        self.title_box.pack_start(cell, True)
        self.title_box.set_size_request(170, 26)
        self.title_box.add_attribute(cell, 'text', 0)
        self.title_id = \
            self.title_box.connect('changed', self.change_track)
        bbox.pack_start(self.title_box, False)

        self.progressbar = gtk.ProgressBar()
        self.progressbar.set_fraction(0)
        self.progressbar.set_size_request(200, -1)
        self.progressbar.set_text(APP.new_progressbar.get_text())
        self.progressbar.props.events = gtk.gdk.BUTTON_MOTION_MASK | \
          gtk.gdk.BUTTON_PRESS_MASK | \
          gtk.gdk.BUTTON_RELEASE_MASK
        self.progressbar.connect('button-press-event', self.seek_begin)
        self.progressbar.connect('button-release-event', self.seek_end)
        self.progressbar.connect('motion-notify-event', self.seek_motion_notify)

        self.progressbar.set_size_request(-1, 24)

        pbox = gtk.VBox()
        label_sizes = 2
        l = gtk.Label()
        l.set_size_request(-1, label_sizes)
        pbox.pack_start(l, False, False)
        pbox.pack_start(self.progressbar, True, True)
        l = gtk.Label()
        l.set_size_request(-1, label_sizes)
        pbox.pack_start(l, False, False)

        bbox.pack_start(pbox, False)

        mm = self.create_button('gtk-fullscreen', toggle_minimode, _('Restore'
            ' Regular View'))
        bbox.pack_start(mm, False)

        main.pack_start(bbox)

        self.add(main)

        self.connect('configure-event', self.on_move)
        self.first = False

    def seek_begin(self, widget, event):
        """
            Starts when seek drag begins
        """
        if not APP.player.current or \
            APP.player.current.type == 'stream': return
        self.seeking = True

    def seek_motion_notify(self, widget, event):
        """
            Simulates dragging on the new progressbar widget
        """
        if not APP.player.current or APP.player.current.type == \
            'stream': return
        mouse_x, mouse_y = event.get_coords()
        progress_loc = self.progressbar.get_allocation()

        value = mouse_x / progress_loc.width
        if value < 0: value = 0
        if value > 1: value = 1
        self.progressbar.set_fraction(value)
        track = APP.player.current

        duration = track.duration
        if duration == -1:
            real = 0
        else:
            real = value * duration
        seconds = real

        remaining_seconds = duration - seconds
        self.progressbar.set_text("%d:%02d / %d:%02d" % ((seconds / 60), 
            (seconds % 60), (remaining_seconds / 60), (remaining_seconds % 60))) 

    def seek_end(self, widget, event):
        """
            Resets seeking flag, actually seeks to the requested location
        """
        mouse_x, mouse_y = event.get_coords()
        progress_loc = self.progressbar.get_allocation()

        value = mouse_x / progress_loc.width
        if value < 0: value = 0
        if value > 1: value = 1

        if not APP.player.current or \
            APP.player.current.type == 'stream':
            self.progressbar.set_fraction(0)
            return
        duration = APP.player.current.duration * gst.SECOND
        if duration == -1:
            real = 0
        else:
            real = value * duration / 100
        seconds = real / gst.SECOND

        duration = APP.player.current.duration
        real = float(value * duration)
        APP.player.seek(real)
        self.seeking = False
        APP.player.current.submitted = True
        APP.emit('seek', real)

    def change_track(self, combo):
        """
            Called when the user uses the title combo to pick a new song
        """
        iter = self.title_box.get_active_iter()
        if iter:
            song = self.model.get_value(iter, 1)
            APP.player.stop()
            APP.player.play_track(song)

    def setup_title_box(self):
        """
            Populates the title box and selects the currently playing track

            The combobox will be populated with all the tracks in the current
            playlist, UNLESS there are more than 50 songs in the playlist.  In
            that case, the current song and 25 songs on either side will
            be displayed. If there aren't enough on one side, it will be
            made up for on the other.
        """
        blank = gtk.ListStore(str, object)
        self.title_box.set_model(blank)
        self.model.clear()
        count = 0; select = -1
        current = APP.player.current
        if current:
            select = 0
        elif APP.songs:
            select = -1
            current = APP.songs[0] 

        count = 0
        select = 0
        for song in APP.songs:
            if song == current and APP.player.current:
                select = count
                break

            count = count + 1

        #A little fiddling to get the first and last songs to display
        #In the combo box
        min_index = max(select - 25, 0)
        max_index = min(min_index + 50, len(APP.songs))
        min_index = max(max_index - 50, 0) #To be safe

        select = select - min_index
        for i in range(min_index, max_index):
            song = APP.songs[i]
            self.model.append([song.title, song])

        self.title_box.set_model(self.model)
        self.title_box.disconnect(self.title_id)
        if select > -1: self.title_box.set_active(select)
        self.title_id = self.title_box.connect('changed',
            self.change_track)
        self.title_box.set_sensitive(len(self.model) > 0)

    def on_move(self, *e):
        """
            Saves the position of the minimode window if it is moved
        """
        (x, y) = self.get_position()
        settings = APP.settings
        settings.set_int('x', x, plugin=plugins.name(__file__))
        settings.set_int('y', y, plugin=plugins.name(__file__))

    def show_window(self):
        """
            Gets the last position from the settings, and then
            displays the mimimode window
        """

        if not self.first:
            self.first = True
            self.show_all()
        else:
            self.show()

        settings = APP.settings
        x = settings.get_int("x", plugin=plugins.name(__file__),   
            default=10)
        y = settings.get_int("y", plugin=plugins.name(__file__),
            default=10)
        self.move(x, y)
        self.setup_title_box()
        self.stick()

        if APP.settings.get_boolean('on_top', plugin=plugins.name(__file__),
            default=False):
            self.set_keep_above(True)
        else:
            self.set_keep_above(False)

        if APP.settings.get_boolean('taskbar',
            plugin=plugins.name(__file__), default=True):
            self.set_property('skip-taskbar-hint', False)
        else:
            self.set_property('skip-taskbar-hint', True)

        if APP.settings.get_boolean('decoration',
            plugin=plugins.name(__file__), default=False):
            self.set_decorated(True)
        else:
            self.set_decorated(False)

    def on_prev(self, button):
        """
            Called when the user presses the previous button
        """

        APP.player.previous()
        self.timeout_cb()

    def on_play(self, button):
        """
            Called when the user clicks the play button
        """

        APP.player.toggle_pause()
        self.timeout_cb()

    def on_stop(self, button=None):
        """
            Called when the user clicks the stop button
        """

        if button: APP.player.stop(True)
        self.timeout_cb()
        self.play.set_image(APP.get_play_image(gtk.ICON_SIZE_MENU))
        self.setup_title_box()
        self.set_title(APP.window.get_title())

    def on_next(self, button):
        """ 
            Called when the user clicks the next button
        """
        
        APP.player.next()
        self.timeout_cb()

    def create_button(self, stock_id, func, tip):
        """
            Creates a little button
        """
        button = gtk.Button()
        button.connect('clicked', func)
        image = gtk.Image()
        image.set_from_stock(stock_id, gtk.ICON_SIZE_MENU)
        button.set_image(image)
        button.set_size_request(26, 26)
        self.tips.set_tip(button, tip)

        return button

    def pause_toggled(self):
        """
            Called when pause is toggled
        """

        track = APP.player.current
        if not track:
            self.play.set_image(APP.get_play_image(gtk.ICON_SIZE_MENU))
        else:
            if APP.player.is_paused():
                self.play.set_image(APP.get_play_image(gtk.ICON_SIZE_MENU))
            else:
                self.play.set_image(APP.get_pause_image(gtk.ICON_SIZE_MENU))
        self.set_title(APP.window.get_title())

    def timeout_cb(self):
        self.progressbar.set_text(APP.new_progressbar.get_text())
        self.progressbar.set_fraction(APP.new_progressbar.get_fraction())
            
        return True

def pause_toggled(exaile, track):
    PLUGIN.pause_toggled()

def play_track(exaile, track):
    PLUGIN.pause_toggled()
    PLUGIN.setup_title_box()

def stop_track(exaile, track):
    PLUGIN.on_stop()

def toggle_hide(*args):
    if not MM_ACTIVE: return False

    if PLUGIN.get_property("visible"):
        PLUGIN.hide()
    else: PLUGIN.show_window()

    return True

def tray_toggled(app, enabled):
    if enabled:
        CONS.connect(app.tray_icon, 'toggle-hide', toggle_hide)
    else:
        CONS.disconnect_object(app.tray_icon)

def pass_func(*args):
    global MM_ACTIVE 
    if PLUGIN.get_property("visible"):
        PLUGIN.hide()
        MM_ACTIVE = False
        APP.window.show()
        return True

def initialize():
    global TIMER_ID, PLUGIN, ACCEL_GROUP, MENU_ITEM, BUTTON

    PLUGIN = MiniWindow()
    TIMER_ID = gobject.timeout_add(1000, PLUGIN.timeout_cb)
    ACCEL_GROUP = gtk.AccelGroup()
    key, mod = gtk.accelerator_parse("<Control><Alt>M")
    ACCEL_GROUP.connect_group(key, mod, gtk.ACCEL_VISIBLE, pass_func)

    APP.window.add_accel_group(ACCEL_GROUP)
    MENU_ITEM = gtk.MenuItem(_("Mini Mode"))
    MENU_ITEM.connect('activate', toggle_minimode)
    MENU_ITEM.add_accelerator('activate', ACCEL_GROUP, key, mod,
        gtk.ACCEL_VISIBLE)
    APP.view_menu.get_submenu().append(MENU_ITEM)
    MENU_ITEM.show()
    PLUGIN.add_accel_group(ACCEL_GROUP)

    BUTTON = gtk.Button(_("Mini Mode"))
    BUTTON.connect('button-release-event', toggle_minimode)
    image = gtk.Image()
    image.set_from_stock(gtk.STOCK_INDEX, gtk.ICON_SIZE_BUTTON)
    BUTTON.set_image(image)

    try:
        BUTTON.set_tooltip_text(_("Switch to Mini Mode"))
    except:
        # Backwards compatibility to GTK < 2.12
        tooltip = gtk.Tooltips()
        tooltip.set_tip(BUTTON, _("Switch to Mini Mode"))

    toolbar = APP.xml.get_widget('top_bar')
    toolbar.pack_end(BUTTON, False)
    
    if APP.settings.get_boolean('switchbutton',
        plugin=plugins.name(__file__), default=False):
        BUTTON.show()
    else:
        BUTTON.hide()

    CONS.connect(APP.player, 'play-track', play_track)
    CONS.connect(APP.player, 'stop-track', stop_track)
    CONS.connect(APP.player, 'pause-toggled', pause_toggled)

    if APP.tray_icon:
        CONS.connect(APP.tray_icon, 'toggle-hide', toggle_hide)
    CONS.connect(APP, 'tray-icon-toggled', tray_toggled)
    return True

def destroy():
    global PLUGIN, MENU_ITEM, ACCEL_GROUP, MENU_ITEM, TIMER_ID, BUTTON

    CONS.disconnect_all()

    if TIMER_ID:
        gobject.source_remove(TIMER_ID)
        TIMER_ID = None

    if PLUGIN:
        PLUGIN.destroy()
        PLUGIN = None

    if MENU_ITEM:
        APP.view_menu.get_submenu().remove(MENU_ITEM)
        MENU_ITEM = None

    if BUTTON:
        APP.xml.get_widget('top_bar').remove(BUTTON)
        BUTTTON = None
        
    if ACCEL_GROUP: 
        APP.window.remove_accel_group(ACCEL_GROUP)
        ACCEL_GROUP = None



# Copyright (C) 2006 Adam Olsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from gettext import gettext as _
import locale, os, re, urllib
import gobject, gtk
from xl import common, xlmisc, media, library
locale.setlocale(locale.LC_ALL, '')
import xl.path

class FilesPanel(object):
    """
        Represents a built in file browser.
    """

    def __init__(self, exaile):
        """
            Expects a Notebook and Exaile instance
        """
        self.exaile = exaile
        self.db = exaile.db
        self.xml = exaile.xml
        self.first_dir = self.exaile.settings.get_str('files_panel_dir',
            xl.path.home)
        self.history = [self.first_dir]

        self.tree = xlmisc.DragTreeView(self, False)
        self.tree.set_headers_visible(True)
        container = self.xml.get_widget('files_box')
        self.scroll = gtk.ScrolledWindow()
        self.scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.scroll.set_shadow_type(gtk.SHADOW_IN)
        self.scroll.add(self.tree)
        container.pack_start(self.scroll, True, True)
        container.show_all()

        self.model = gtk.ListStore(gtk.gdk.Pixbuf, str, str)
        self.tree.set_model(self.model)
        selection = self.tree.get_selection()
        selection.set_mode(gtk.SELECTION_MULTIPLE)
        self.directory = xlmisc.get_icon('gnome-fs-directory')
        self.track = gtk.gdk.pixbuf_new_from_file(xl.path.get_data('images',
            'track.png'))
        self.up = self.xml.get_widget('files_up_button')
        self.up.connect('clicked', self.go_up)
        self.back = self.xml.get_widget('files_back_button')
        self.back.connect('clicked', self.go_prev)
        self.next = self.xml.get_widget('files_next_button')
        self.next.connect('clicked', self.go_next)
        self.entry = self.xml.get_widget('files_entry')
        self.entry.connect('activate', self.entry_activate)
        self.xml.get_widget('files_refresh_button').connect('clicked',
            self.refresh)
        self.counter = 0

        pb = gtk.CellRendererPixbuf()
        text = gtk.CellRendererText()
        # TRANSLATORS: Filename column in the file browser
        col = gtk.TreeViewColumn(_('Filename'))
        col.pack_start(pb, False)
        col.pack_start(text, True)
        col.connect('notify::width', self.set_column_width)

        width = self.exaile.settings.get_int('ui/files_%s_col_width' %
            _('Filename'), 130)

        col.set_fixed_width(width)
        col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
        col.set_resizable(True)
        col.set_attributes(pb, pixbuf=0)
        col.set_attributes(text, text=1)
        self.i = 0

        self.tree.append_column(col)

        width = self.exaile.settings.get_int('ui/files_%s_col_width' %
            _('Size'), 50)

        text = gtk.CellRendererText()
        text.set_property('xalign', 1.0)
        # TRANSLATORS: Filesize column in the file browser
        col = gtk.TreeViewColumn(_('Size'))
        col.set_fixed_width(width)
        col.set_resizable(True)
        col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
        col.pack_start(text, False)
        col.set_attributes(text, text=2)
        col.connect('notify::width', self.set_column_width)
        self.tree.append_column(col)

        # set up the search entry
        self.search = self.xml.get_widget('files_search_entry')
        self.search.connect('key-release-event', self.key_release)
        self.search.connect('activate', lambda *e:
            self.load_directory(self.current, history=False,
            keyword=unicode(self.search.get_text(), 'utf-8')))

        self.key_id = None

        self.load_directory(self.first_dir, False)
        self.tree.connect('row-activated', self.row_activated)
        self.menu = xlmisc.Menu()
        self.menu.append(_("Append to Playlist"), self.append)
        self.queue_item = self.menu.append(_("Queue Items"), self.append)

    def set_column_width(self, col, stuff=None):
        """
            Called when the user resizes a column
        """
        name = "ui/files_%s_col_width" % col.get_title()
        self.exaile.settings[name] = col.get_width()

    def key_release(self, *e):
        """
            Called when someone releases a key.
            Sets up a timer to simulate live-search
        """
        if self.key_id:
            gobject.source_remove(self.key_id)
            self.key_id = None

        self.key_id = gobject.timeout_add(700, lambda *e:
            self.load_directory(self.current, history=False,
            keyword=unicode(self.search.get_text(), 'utf-8')))

    def drag_get_data(self, treeview, context, sel, target_id, etime):
        """
            Called when a drag source wants data for this drag operation
        """

        songs = self.get_selected_songs()
        uris = [urllib.quote(song.loc.encode(xlmisc.get_default_encoding())) for song in songs]

        sel.set_uris(uris)

    def button_press(self, widget, event):
        """
            Called to show the menu when someone right clicks
        """
        selection = self.tree.get_selection()
        if event.button == 3:
            self.menu.popup(None, None, None, event.button, event.time)
            return True
        if selection.count_selected_rows() <= 1: return False

    def append(self, widget, event):
        self.exaile.status.set_first(_("Scanning and adding files..."))
        songs = self.get_selected_songs()
        if songs:
            self.exaile.playlist_manager.append_songs(songs, queue=(widget == self.queue_item),
                play=False)
        self.counter = 0
        self.exaile.status.set_first(None)

    def get_selected_songs(self):
        """
            Appends recursively the selected directory/files
        """
        selection = self.tree.get_selection()
        (model, paths) = selection.get_selected_rows()
        songs = library.TrackData()

        for path in paths:
            iter = self.model.get_iter(path)
            value = unicode(model.get_value(iter, 1), 'utf-8')
            value = os.path.join(self.current, value)
            (stuff, ext) = os.path.splitext(value)
            if os.path.isdir(value):
                self.append_recursive(songs, value)
            elif ext.lower() in media.SUPPORTED_MEDIA:
                tr = self.get_track(value)
                if tr:
                    songs.append(tr)

        if songs:
            # sort the songs
            ar = [(song.artist, song.album, song.track, song.title, song)
                for song in songs]
            ar.sort()
            songs = [item[-1] for item in ar]
            return songs
        else:
            return None

    def get_track(self, path):
        """
            Gets a track
        """
        tr = library.read_track(self.exaile.db, self.exaile.all_songs, path)
        return tr

    def append_recursive(self, songs, dir):
        """
            Appends recursively
        """
        for file in os.listdir(dir):
            if os.path.isdir(os.path.join(dir, file)):
                self.append_recursive(songs, os.path.join(dir, file))
            else:
                (stuff, ext) = os.path.splitext(file)
                if ext.lower() in media.SUPPORTED_MEDIA:
                    tr = self.get_track(os.path.join(dir, file))
                    if tr:
                        songs.append(tr)
                if self.counter >= 15:
                    xlmisc.finish()
                    self.counter = 0
                else:
                    self.counter += 1

    def refresh(self, widget):
        """
            Refreshes the current view
        """
        self.load_directory(self.current, False)

    def entry_activate(self, widget, event=None):
        """
            Called when the user presses enter in the entry box
        """
        dir = os.path.expanduser(unicode(self.entry.get_text(), 'utf-8'))
        if not os.path.isdir(dir):
            self.entry.set_text(self.current)
            return
        self.load_directory(dir)

    def go_next(self, widget):
        """
            Goes to the next entry in history
        """
        self.i += 1 
        self.load_directory(self.history[self.i], False)
        if self.i >= len(self.history) - 1:
            self.next.set_sensitive(False)
        if len(self.history):
            self.back.set_sensitive(True)
            
    def go_prev(self, widget):
        """
            Previous entry
        """
        self.i -= 1
        self.load_directory(self.history[self.i], False)
        if self.i == 0:
            self.back.set_sensitive(False)
        if len(self.history):
            self.next.set_sensitive(True)

    def go_up(self, widget):
        """
            Moves up one directory
        """
        cur = re.sub('(.*)%s[^%s]*$' % (os.sep, os.sep),
            r'\1', self.current)
        if not cur: cur = os.sep

        self.load_directory(cur)

    def row_activated(self, *i):
        """
            Called when someone double clicks a row
        """
        selection = self.tree.get_selection()
        (model, paths) = selection.get_selected_rows()

        for path in paths:
            iter = self.model.get_iter(path)
            value = unicode(model.get_value(iter, 1), 'utf-8')
            dir = os.path.join(self.current, value)
            if os.path.isdir(dir):
                self.load_directory(dir)
            else:
                if common.any(dir.endswith(ext) for ext in xlmisc.PLAYLIST_EXTS):
                    self.exaile.import_playlist(dir, True)
                else:
                    tr = library.read_track(self.exaile.db, self.exaile.all_songs,
                        dir)
                    if tr:
                        self.exaile.playlist_manager.append_songs((tr, ))

    def load_directory(self, dir, history=True, keyword=None):
        """
            Loads a directory into the files view
        """
        try:
            paths = os.listdir(dir)
        except OSError:
            paths = os.listdir(xl.path.home)

        self.exaile.settings.set_str('files_panel_dir', dir)
        self.current = dir
        directories = []
        files = []
        for path in paths:
            if path.startswith('.'): continue

            if keyword and path.lower().find(keyword.lower()) == -1:
                continue
            full = os.path.join(dir, path)
            if os.path.isdir(full):
                directories.append(path)

            else:
                (stuff, ext) = os.path.splitext(path)
                if ext.lower() in media.SUPPORTED_MEDIA:
                    files.append(path)

        directories.sort()
        files.sort()

        self.model.clear()
        
        for d in directories:
            self.model.append([self.directory, d, '-'])

        for f in files:
            try:
                info = os.stat(os.path.join(dir, f))
            except OSError:
                continue
            size = info[6]
            size = size / 1024
            # The next two lines are equivalent to
            # locale.format_string(_("%d KB"), size, True)
            # which is only available in Python >=2.5.
            size = locale.format('%d', size, True)
            size = _("%s KB") % size

            self.model.append([self.track, f, size])

        self.tree.set_model(self.model)
        self.entry.set_text(self.current)
        if history: 
            self.back.set_sensitive(True)
            self.history = self.history[:self.i + 1]
            self.history.append(self.current)
            self.i = len(self.history) - 1
            self.next.set_sensitive(False)



# Copyright (C) 2006 Adam Olsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import pygst, gtk
pygst.require('0.10')
import gst, gobject, random, time, re, os, md5
import thread
from gettext import gettext as _
from xl import xlmisc, common, media, library, logger
from xl.gui import playlist as playlistgui, information
import xl.path

class Player(gobject.GObject):
    """
        This is the main player interface, other engines will subclass it
    """

class GSTPlayer(Player):
    """
        Gstreamer engine
    """

    def __init__(self):
        Player.__init__(self)
        self.playing = False
        self.connections = []
        self.last_position = 0
        self.eof_func = None
        self.tag_func = None
        self.setup_playbin()

    def setup_playbin(self):
        self.playbin = gst.element_factory_make('playbin')
        self.bus = self.playbin.get_bus()
        self.bus.add_signal_watch()
        self.bus.enable_sync_message_emission()
        self.audio_sink = None

    def set_volume(self, vol):
        """
            Sets the volume for the player
        """
        self.playbin.set_property('volume', vol)

    def _get_gst_state(self):
        """
            Returns the raw GStreamer state
        """
        return self.playbin.get_state(timeout=50*gst.MSECOND)[1]

    def get_state(self):
        """
            Returns the player state: 'playing', 'paused', or 'stopped'.
        """
        state = self._get_gst_state()
        if state == gst.STATE_PLAYING:
            return 'playing'
        elif state == gst.STATE_PAUSED:
            return 'paused'
        else:
            return 'stopped'

    def is_playing(self):
        """
            Returns True if the player is currently playing
        """
        return self._get_gst_state() == gst.STATE_PLAYING

    def is_paused(self):
        """
            Returns True if the player is currently paused
        """
        return self._get_gst_state() == gst.STATE_PAUSED

    def on_message(self, bus, message, reading_tag = False):
        """
            Called when a message is received from gstreamer
        """
        if message.type == gst.MESSAGE_TAG and self.tag_func:
            self.tag_func(message.parse_tag())
        elif message.type == gst.MESSAGE_EOS and not self.is_paused() \
            and self.eof_func:
            self.eof_func()
        elif message.type == gst.MESSAGE_ERROR:
            print message, dir(message)

        return True

    def __notify_source(self, o, s, num):
        s = self.playbin.get_property('source')
        s.set_property('device', num)
        self.playbin.disconnect(self.notify_id)

    def play(self, uri):
        """
            Plays the specified uri
        """
        if not self.audio_sink:
            self.set_audio_sink('')

        if not self.connections and not self.is_paused() and not \
            uri.find("lastfm://") > -1:

            self.connections.append(self.bus.connect('message', self.on_message))
            self.connections.append(self.bus.connect('sync-message::element',
                self.on_sync_message))

            if '://' not in uri: 
                if not os.path.isfile(uri):
                    raise Exception('File does not exist: ' + uri)
                uri = 'file://%s' % uri # FIXME: Wrong.
            uri = uri.replace('%', '%25')

            # for audio cds
            if uri.startswith("cdda://"):
                num = uri[uri.find('#') + 1:]
                uri = uri[:uri.find('#')]
                self.notify_id = self.playbin.connect('notify::source',
                    self.__notify_source, num)

            self.playbin.set_property('uri', uri.encode(xlmisc.get_default_encoding()))

        self.playbin.set_state(gst.STATE_PLAYING)

    def on_sync_message(self, bus, message):
        """
            called when gstreamer requests a video sync
        """
        if message.structure.get_name() == 'prepare-xwindow-id' and \
            VIDEO_WIDGET:
            xlmisc.log('Gstreamer requested video sync')
            VIDEO_WIDGET.set_sink(message.src)

    def seek(self, value, wait=True):
        """
            Seeks to a specified location (in seconds) in the currently
            playing track
        """
        value = int(gst.SECOND * value)

        if wait: self.playbin.get_state(timeout=50*gst.MSECOND)
        event = gst.event_new_seek(1.0, gst.FORMAT_TIME,
            gst.SEEK_FLAG_FLUSH|gst.SEEK_FLAG_ACCURATE,
            gst.SEEK_TYPE_SET, value, gst.SEEK_TYPE_NONE, 0)

        res = self.playbin.send_event(event)
        if res:
            self.playbin.set_new_stream_time(0L)
        else:
            xlmisc.log("Couldn't send seek event")
        if wait: self.playbin.get_state(timeout=50*gst.MSECOND)

        self.last_seek_pos = value

    def pause(self):
        """
            Pauses the currently playing track
        """
        self.playbin.set_state(gst.STATE_PAUSED)

    def toggle_pause(self):
        if self.is_paused():
            self.playbin.set_state(gst.STATE_PLAYING)
        else:
            self.playbin.set_state(gst.STATE_PAUSED)

    def stop(self):
        """
            Stops the playback of the currently playing track
        """
        for connection in self.connections:
            self.bus.disconnect(connection)
        self.connections = []

        self.playbin.set_state(gst.STATE_NULL)

    def get_position(self):
        """
            Gets the current playback position of the playing track
        """
        if self.is_paused(): return self.last_position
        try:
            self.last_position = \
                self.playbin.query_position(gst.FORMAT_TIME)[0]
        except gst.QueryError:
            self.last_position = 0

        return self.last_position

class ExailePlayerErrorWindow(gtk.Window):
    """
        A simple log that will track errors during playback.  
        This was developed to catch errors like "file was not found" and go to
        the next song instead of interupting playback
    """
    def __init__(self, parent):
        """
            Initializes the window
        """
        gtk.Window.__init__(self)
        self.set_title(_('Errors'))
        self.set_transient_for(parent)
        self.set_position(gtk.WIN_POS_CENTER_ON_PARENT)

        main = gtk.VBox()
        self.connect('delete_event', self.on_delete)
        main.set_border_width(3)
        main.set_spacing(3)
        label = gtk.Label()
        label.set_markup('<b>' + _('The following errors have occurred') +
            '</b>')
        main.pack_start(label, False, False)
        self.add(main)
        self.scroll = gtk.ScrolledWindow()
        self.scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.scroll.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        main.pack_start(self.scroll, True, True)

        self.view = gtk.TextView()
        self.view.set_editable(False)
        self.buf = self.view.get_buffer()
        self.scroll.add(self.view)

        buttons = gtk.HBox()
        close = gtk.Button(stock=gtk.STOCK_CLOSE)
        buttons.pack_end(close, False, False)
        main.pack_start(buttons, False, False)

        close.connect('clicked', lambda *e: self.hide())
        self.resize(400, 300)

    def log(self, message, timestamp=None):
        """ 
            Logs a message
        """

        if not timestamp: timestamp = time.time()
        lt = time.localtime(timestamp)
        text = "[%s] %s\n" % (time.strftime("%H:%M:%S", lt), message)
        char = self.buf.get_char_count()
        iter = self.buf.get_iter_at_offset(char + 1)

        self.buf.insert(iter, text)

        char = self.buf.get_char_count()
        iter = self.buf.get_iter_at_offset(char + 1)
        self.view.scroll_to_iter(iter, 0)

    def on_delete(self, *e):
        """
            Hides the window instead of destroying it
        """
        self.hide()

        return True

class ExailePlayer(GSTPlayer):
    """
        Exaile interface to the GSTPlayer
    """
    __gsignals__ = {
        'play-track': (gobject.SIGNAL_RUN_LAST, None, (media.Track,)),
        'stop-track': (gobject.SIGNAL_RUN_LAST, None, (media.Track,)),
        'pause-toggled': (gobject.SIGNAL_RUN_LAST, None, (media.Track,))
    }

    def __init__(self, exaile):
        self.exaile = exaile
        GSTPlayer.__init__(self)
        self.last_played = None
        self.played = []
        self.queued = []
        self.history = []
        self._stop_track = None
        self.next_track = None
        self.shuffle = False
        self.repeat = False
        self.last_track = ''
        self.equalizer = None
        self.error_window = ExailePlayerErrorWindow(self.exaile.window)

        self.eof_func = self.next
        self.current = None

        self._sink_element_factories = []
        self.add_sink_element_factory(ReplayGainElementFactory)
        self.add_sink_element_factory(EqualizerElementFactory)

    def add_sink_element_factory(self, factory):
        """
            adds a GstElement factory to the audio sink link
        """
        self._sink_element_factories.append(factory)
        # TODO: Check if GstElements are unreferenced and cleaned up
        # TODO: Redesign GstElement management
        self.audio_sink = None

    def del_sink_element_factory(self, factory):
        """
            deletes a GstElement factory from the audio sink link
        """
        self._sink_element_factories.remove(factory)
        self.audio_sink = None

    def get_stop_track(self):
        """
            returns the stop_track (track to stop playback)
        """
        return self._stop_track

    def set_stop_track(self, value):
        """
            sets the stop_track (track to stop playback)
        """
        stop_button = self.exaile.stop_track_button
        if value:
            stop_button.set_sensitive(True)
        else:
            stop_button.set_sensitive(False)
        self._stop_track = value
    
    stop_track = property(get_stop_track, set_stop_track)

    def set_audio_sink(self, sink=None):
        """
            Sets the audio sink up.  It tries the passed in value, and if that
            doesn't work, it tries autoaudiosink
        """

        self.audio_sink = self._create_sink(sink)

        # if the audio_sink is still not set, use a fakesink
        if not self.audio_sink:
            xlmisc.log('Audio sink could not be set up.  Using a fakesink '
               'instead.  Audio will not be available.')
            self.audio_sink = gst.element_factory_make('fakesink')

        self.playbin.set_property('audio-sink', self.audio_sink)

    def _create_sink(self, sink=None):
        """
            Creates an element: equalizer -> replaygain -> named sink.

            If the named sink is None, use the audio_sink setting.
            The equalizer and ReplayGain elements are optional and will not be
            created if they don't exist or are disabled.
        """

        if not sink: sink = self.exaile.settings.get_str('audio_sink',
            'Use GConf Settings')
        sink = sink.lower()
        if "gconf" in sink: sink = 'gconfaudiosink'
        elif "auto" in sink: sink = 'autoaudiosink'
        try:
            asink = gst.element_factory_make(sink)
            if sink == 'gconfaudiosink':
                # The "Music and Movies" profile
                asink.set_property('profile', 1)
        except:
            xlmisc.log("Could not create sink %s.  Trying autoaudiosink." %
                sink)
            asink = gst.element_factory_make('autoaudiosink')

        sinkbin = gst.Bin()
        sink_elements = []

        # iterate through sink element factory list
        for element_factory in self._sink_element_factories:
            if element_factory.is_enabled(self):
                # This should be made a try: except: statement in case creation fails
                sink_elements += element_factory.get_elements(self)
                xlmisc.log(element_factory.name + " support initialized.")
            else:
                xlmisc.log("Not using " + element_factory.name + " disabled by the user")

        # if still empty just use asink and end
        if not sink_elements:
            return asink

        # otherwise put audiosink as last element
        sink_elements.append(asink)

        # add elements to sink and link them
        sinkbin.add(*sink_elements)
        gst.element_link_many(*sink_elements)

        # create sink pad in that links to sink pad of first element
        sinkpad = sink_elements[0].get_static_pad('sink')
        sinkbin.add_pad(gst.GhostPad('sink', sinkpad))

        return sinkbin

    def get_position(self):
        """
            Gets current position 
        """
        position = GSTPlayer.get_position(self)

        return position

    def setup_playbin(self):
        GSTPlayer.setup_playbin(self)
        self.playbin.set_property('volume',
            self.exaile.settings.get_float('volume', .7))

    def set_volume(self, volume):
        GSTPlayer.set_volume(self, volume)

    def toggle_pause(self):
        """
            Toggles pause.  If it's a streaming track, it stops the stream and
            restarts it
        """
        track = self.current

        if not track:
            self.play()
            return

        if self.is_paused():
            if self.current.type == 'stream':
                self.playbin.set_state(gst.STATE_READY)
                self.playbin.set_state(gst.STATE_PLAYING)
                return
            self.exaile.play_button.set_image(self.exaile.get_pause_image())
        else:
            self.exaile.play_button.set_image(self.exaile.get_play_image())
        GSTPlayer.toggle_pause(self)

        if self.exaile.tracks: self.exaile.tracks.queue_draw()
        self.emit('pause-toggled', self.current)

    def get_next_queued(self):
        if self.next_track == None:
            self.next_track = self.current
        track = self.queued[0]
        if not track in self.exaile.tracks.songs:
            self.exaile.tracks.append_song(track)
        self.queued = self.queued[1:len(self.queued)]
        xlmisc.log('Playing queued track "%s"' % track)

        return track

    def get_next_shuffle_track(self):
        """
            Returns the next random track that hasn't already been played
        """
        count = 0

        while True:
            if len(self.exaile.songs) == 0: return
            current_index = random.randint(0, len(self.exaile.songs) -
                1)
            track = self.exaile.tracks.songs[current_index]
            if count >= 500 or track not in \
                self.played:
                
                if track in self.played:
                    for track in self.exaile.tracks.songs:
                        if not track in self.played:
                            break
                    self.played = []
                    if not self.repeat:
                        return

                break

            count += 1

        return track

    def get_current_position(self):
        """
            Gets the current position relative to the playing track's length
        """
        value = 0
        duration = self.current.duration * gst.SECOND
        if duration:
            value = self.get_position() * 100.0 / duration

        return value

    def find_stream_uri(self, track):
        title = track.album[:30]
        if not title: title = None
        self.exaile.playlist_manager.import_playlist(track.io_loc, 
            play=True, title=title, newtab=True)

    def play_track(self, track, from_button=False, ret=True):
        """
            Plays a track. If from_button is True, it means the user double
            clicked on a track or pressed the play button.  In this case if
            there is an error, playback will be stopped, otherwise, it will
            move on to the next track
        """
        self.stop(False)
        lowtrackloc = track.loc.lower()
        if common.any(lowtrackloc.endswith(ext) for ext in xlmisc.PLAYLIST_EXTS):
            self.find_stream_uri(track)
            if ret: return True

        if track.type == 'stream':
            track.start_time = time.time()

#        if track.type == 'podcast':
#            if not track.download_path:
#                common.error(self.exaile.window, _('Podcast has not yet been '
#                    'downloaded'))
#                return

        self.exaile.play_button.set_image(self.exaile.get_pause_image())

        try:
            play_loc = track.loc
            if track.type == 'podcast' and track.download_path:
                play_loc = track.download_path

            # custom track setup
            if hasattr(track, 'start_play'):
                track.start_play()

            GSTPlayer.play(self, play_loc)
        except Exception, e:
            logger.log_exception()
            self.error_window.log(str(e))
            self.error_window.show_all()
            if from_button: self.exaile.player.stop()
            else: 
                self.stop()
                self.current = track
                self.next()
            if ret: return False

        self.current = track
        self.exaile.update_track_information()

        self.emit('play-track', track)

        # update information tab if it's open and set to do so
        if self.exaile.settings.get_boolean('ui/information_autoswitch',
            False):
            information.update_info(self.exaile.playlists_nb, track)

        artist_id = library.get_column_id(self.exaile.db, 'artists', 'name', track.artist)
        library.get_album_id(self.exaile.db, artist_id, track.album)

        if track.type != 'stream':
            self.exaile.cover_manager.fetch_cover(track)

        self.exaile.show_osd()
        if self.exaile.tracks: self.exaile.tracks.queue_draw()

        if self.exaile.settings.get_boolean('ui/ensure_visible', False):
            self.exaile.goto_current()

        playlistgui.update_queued(self.exaile)

        # if we're in dynamic mode, find some tracks to add
        if self.exaile.dynamic.get_active():
            thread.start_new_thread(self.exaile.get_suggested_songs, tuple())

        track.last_played = time.strftime("%Y-%m-%d %H:%M:%S",
            time.localtime())

        path_id = library.get_column_id(self.exaile.db, 'paths', 'name', track.loc)
        self.exaile.db.execute("UPDATE tracks SET last_played=? WHERE path=?",
            (track.last_played, track.loc))
        self.exaile.rewind_track = 0
        if ret: return True

    def play(self):
        """
            Plays the currently selected track
        """
        try:
            self.stop()

            if self.exaile.tracks == None: 
                self.exaile.tracks = self.exaile.playlists_nb.get_nth_page(0)
                self.exaile.songs = self.exaile.tracks.songs

            self.last_played = self.current

            track = self.exaile.tracks.get_selected_track()
            if not track:
                if self.exaile.tracks.songs:
                    self.next()
                    return
                return

            if track in self.queued: del self.queued[self.queued.index(track)]
            self.current = track

            self.played = [track]
            self.history.append(track)
            self.play_track(track, from_button=True)
        except Exception, e:
            xlmisc.log_exception()
            common.error(self.exaile.window, str(e))
            self.stop()
        
    def next(self):
        """
            Plays the next track in the playlist
        """
        if self.current != None:
            if self.get_current_position() < 50:
                self.exaile.update_rating(self.current, rating=-1)

        self.stop(False)
        if self.exaile.tracks == None: return
        if self.stop_track:
            if self.current == self.stop_track:
                self.stop_track = None
                self.stop()
                return
      
        track = self.exaile.tracks.get_next_track(self.current)

        if not track:
            if not self.exaile.tracks.get_songs():
                if not self.queued: return
            else: track = self.exaile.tracks.get_songs()[0]

        if self.next_track != None and not self.queued:
            track = self.exaile.tracks.get_next_track(self.next_track)
            if not track: track = self.exaile.tracks.get_songs()[0]
            self.next_track = None

        # for queued tracks
        if len(self.queued) > 0:
            track = self.get_next_queued()
            self.play_track(track)
            self.current = track
            self.played.append(track)
            return
        else:
            # for shuffle mode
            if self.shuffle:
                track = self.get_next_shuffle_track()

        if not self.shuffle and \
            not self.exaile.tracks.get_next_track(self.current):
            if self.repeat:
                self.current = self.exaile.tracks.get_songs()[0]
            else:
                self.played = []
                self.current = None
                return
        
        self.history.append(track)
        self.played.append(track)

        if track: 
            if not self.play_track(track):
                return
        self.current = track
        if self.exaile.tracks: self.exaile.tracks.queue_draw()

    def previous(self):
        """
            Go to the previous track
        """

        # if the current track has been playing for less than 5 seconds, 
        # just restart it
        if self.exaile.rewind_track >= 4:
            track = self.current
            if track:
                self.stop()
                self.current = track
                self.play_track(track)
                self.exaile.rewind_track = 0
            return

        if self.history:
            self.stop()
            track = self.history.pop()
        else:
            track = self.exaile.tracks.get_previous_track(self.current)
        if not track: track = self.exaile.songs[0]

        self.current = track
        self.play_track(track)
        if self.exaile.tracks:  self.exaile.tracks.queue_draw()

    def stop(self, reset_current=True):
        """
            Stops the currently playing track
        """
        exaile = self.exaile

        exaile.status.set_first(None)
        exaile.cover.set_image(xl.path.get_data('images', 'nocover.png'))
        exaile.cover_manager.stop_cover_thread()

        if self.current: self.current.start_time = 0
        current = self.current
        self.emit('stop-track', current)
        if reset_current: self.current = None
        GSTPlayer.stop(self)

        exaile.playing = False

        title = _('Exaile Music Player')
        exaile.window.set_title(title)
        if exaile.tray_icon:
            exaile.tray_icon.set_tooltip(title)

        exaile.play_button.set_image(exaile.get_play_image())
        if exaile.tracks: exaile.tracks.queue_draw()
        exaile.update_track_information(None)
        exaile.new_progressbar.set_text('0:00 / 0:00')

class ReplayGainElementFactory(object):
    name = u"ReplayGain"
    
    @staticmethod
    def is_enabled(exaileplayer):
        return exaileplayer.exaile.settings.get_boolean('replaygain/enabled', True)
    
    @staticmethod
    def get_elements(exaileplayer):
        replaygain = None

        try:
            replaygain = gst.element_factory_make('rgvolume')
        except gst.PluginNotFoundError:
            xlmisc.log("ReplayGain support requires gstreamer-plugins-bad 0.10.5")

        if replaygain:
            replaygain.set_property('album-mode',
                exaileplayer.exaile.settings.get_boolean('replaygain/album_mode', True))
            replaygain.set_property('pre-amp',
                exaileplayer.exaile.settings.get_float('replaygain/preamp'))
            replaygain.set_property('fallback-gain',
                exaileplayer.exaile.settings.get_float('replaygain/fallback'))
            
            # Using the ugly method for returing a 1-element tuple
            return replaygain,

        return ()

class EqualizerElementFactory(object):
    name = u"Equalizer"
    
    @staticmethod
    def is_enabled(exaileplayer):
        if not exaileplayer.exaile.settings.get_boolean('equalizer/enabled',
            False): 
            return False
        return not exaileplayer.exaile.options.noeq
    
    @staticmethod
    def get_elements(exaileplayer):
        try: # Equalizer element is still not very common 
            exaileplayer.equalizer = gst.element_factory_make('equalizer-10bands')
        except gst.PluginNotFoundError:
            xlmisc.log("Equalizer support requires gstreamer-plugins-bad 0.10.5")

        elements = []
        if exaileplayer.equalizer:
            elements.append(exaileplayer.equalizer)
            elements.append(gst.element_factory_make('audioconvert'))

            bands = exaileplayer.exaile.settings.get_list('equalizer/band-values', [0] * 10)
            for i, v in enumerate(bands):
                exaileplayer.equalizer.set_property(('band' + str(i)), v)
        return elements

# VideoWidget and VideoArea code taken from Listen media player
# http://listen-gnome.free.fr
class VideoWidget(gtk.Window):
    def __init__(self, exaile):
        gtk.Window.__init__(self)
        self.exaile = exaile
        self.imagesink = None
        self.area = VideoArea()
        self.is_fullscreen = False
        self.ebox = gtk.EventBox()
        self.ebox.add(self.area)
        self.ebox.connect('button-press-event', self.button_press)
        self.add(self.ebox)
        self.resize(700, 500)
        self.loaded = False
        self.connect('delete_event', self.on_delete)
        self.set_title(_("Exaile Music Player"))

    def button_press(self, widget, event):
        """
            Called when the user clicks on the event box
        """
        if event.type == gtk.gdk._2BUTTON_PRESS:
            self.toggle_fullscreen()

    def toggle_fullscreen(self):
        """
            Toggles fullscreen mode
        """
        if not self.is_fullscreen:
            self.is_fullscreen = True
            self.fullscreen()
            self.present()
        else:
            self.is_fullscreen = False
            self.unfullscreen()
            self.present()

    def on_delete(self, *e):
        """
            Called when the window is closed
        """
        global VIDEO_WIDGET
        self.hide()
        VIDEO_WIDGET = None

        track = self.exaile.player.current
        position = 0
        play_track = False
        if track is not None and self.exaile.player.is_playing():
            try:
                position = self.exaile.player.playbin.query_position(gst.FORMAT_TIME)[0] 
            except gst.QueryError:
                position = 0
            self.exaile.player.stop(False)
            play_track = True

        self.exaile.player.setup_playbin()
        xlmisc.finish()

        self.exaile.player.playbin.get_state(timeout=50*gst.MSECOND)
        if play_track:
            self.exaile.player.play_track(track)
            if position and track.type != 'stream':
                self.exaile.player.seek(float(position / gst.SECOND))

        return True

    def set_sink(self, sink):
        self.imagesink = sink

        self.set_window_id()
        self.area.set_sink(sink)

        # workaround to launch the visualisation on startup
        # And prevent the "Xerror GC bad" problem when 
        # visualisation start and widget not completey realize
        if not self.loaded:
            self.child.child.do_expose_event(None)
            self.loaded = True

    def set_window_id(self):
        self.imagesink.set_xwindow_id(self.child.child.window.xid)

class VideoArea(gtk.DrawingArea):
    def __init__(self,imagesink=None):
        gtk.DrawingArea.__init__(self)
        self.unset_flags(gtk.DOUBLE_BUFFERED)
        self.imagesink = imagesink

    def set_sink(self,imagesink):
        self.imagesink = imagesink

    def do_expose_event(self, event):
        if self.imagesink:
            self.imagesink.expose()
            return False
        else:
            return True

VIDEO_WIDGET = None
def show_visualizations(exaile):
    """
        Shows the visualizations window
    """
    global VIDEO_WIDGET
    xlmisc.log('Enabling visualizations')
    if VIDEO_WIDGET:
        VIDEO_WIDGET.present()
        return
    track = exaile.player.current
    play_track = False
    position = 0
    if track is not None and exaile.player.is_playing():
        try:
            position = exaile.player.playbin.query_position(gst.FORMAT_TIME)[0] 
        except gst.QueryError:
            position = 0
        exaile.player.stop(False)
        play_track = True

    VIDEO_WIDGET = VideoWidget(exaile)
    VIDEO_WIDGET.show_all()
    xlmisc.finish()
    exaile.player.playbin.get_state(timeout=50*gst.MSECOND)
    video_sink = gst.element_factory_make('xvimagesink')
    vis = gst.element_factory_make('goom')
    exaile.player.playbin.set_property('video-sink', video_sink)
    exaile.player.playbin.set_property('vis-plugin', vis)

    xlmisc.log("Player position is %d" % (position / gst.SECOND))
    if play_track:
        exaile.player.play_track(track)
        exaile.player.current = track
        if position and track.type != 'stream':
            exaile.player.seek(position / gst.SECOND, False)

    return True



import mutagen, mutagen.id3, mutagen.trueaudio
from xl import xlmisc

IDS = { "TIT2": "title",
        "TPE1": "artist",
        "TALB": "album",
        "TRCK": "track",
        "TDRC": "year",
        "TCON": "genre"
        }

SDI = dict([(v, k) for k, v in IDS.iteritems()])

def get_tag(id3, t):
    """
        Reads a specific id3 tag from the file
    """
    if not id3.has_key(t): return ""
    text = unicode(id3[t])

    text = text.replace('\n', ' ').replace('\r', ' ')
    return text

def write_tag(tr):
    try:
        id3 = mutagen.id3.ID3(tr.io_loc)
    except mutagen.id3.ID3NoHeaderError:
        id3 = mutagen.id3.ID3()

    for key, id3name in SDI.items():
        id3.delall(id3name)

    for k, v in IDS.iteritems():
        if k == 'TRCK': continue

        try:
            frame = mutagen.id3.Frames[k](encoding=3,
                text = unicode(getattr(tr, v)))
            id3.loaded_frame(frame)
        except:
            xlmisc.log_exception()

        if tr.track > -1:
            track = str(tr.track)
            if tr.disc_id > -1:
                track = "%s/%s" % (track, tr.disc_id)

            frame = mutagen.id3.Frames['TRCK'](encoding=3,
                text=track)

            id3.loaded_frame(frame)

        id3.save(tr.io_loc)    

def fill_tag_from_path(tr):
    info = mutagen.trueaudio.TrueAudio(tr.io_loc)
    tr.length = info.info.length

    try:    
        id3 = mutagen.id3.ID3(tr.io_loc)
        tr.title = get_tag(id3, 'TIT2')
        tr.artist = get_tag(id3, 'TPE1')
        tr.album = get_tag(id3, 'TALB')
        tr.genre = get_tag(id3, 'TCON')

        trackinfo = get_tag(id3, 'TRCK')
        if '/' in trackinfo:
            tr.track, tr.disc_id = trackinfo.split('/')

            try:
                tr.track = int(tr.track)
            except ValueError: tr.track = -1

            try:
                tr.disc_id = int(tr.disc_id)
            except ValueError: tr.disc_id = -1
        else:
            try:
                tr.track = int(trackinfo)
            except ValueError:
                tr.track = -1

        tr.year = get_tag(id3, 'TDRC')

    except OverflowError:
        pass
    except mutagen.id3.ID3NoHeaderError:
        pass
    except IOError:
        pass
    except:
        xlmisc.log_exception()



#!/usr/bin/env python

import os, sys, glob

command = None
try:
    command = sys.argv[1]
except IndexError:
    pass

os.chdir('po')
os.system('intltool-update --pot --gettext-package=messages --verbose')

if command != 'compile':
    print "\n\n**********\n"
    print "Now edit messages.pot, save it as <locale>.po, and post it on\n" \
        "our but tracker (https://bugs.launchpad.net/exaile/)\n\n" \
        "Thank you!"

else:

    files = glob.glob('*.po')
    for f in files:
        l = os.path.splitext(f)
        os.system('mkdir -p -m 0777 %s/LC_MESSAGES' % l[0])

        print "Generating translation for %s locale" % l[0]
        os.system('msgmerge -o - %s messages.pot | msgfmt -c -o %s/LC_MESSAGES/exaile.mo -' % (f, l[0]))

os.chdir('..')



#!/usr/bin/env python
# Copyright (C) 2006 Adam Olsen <arolsen@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import gtk, gobject, re, time
from xl import library, media, common, xlmisc
import xl, os
from gettext import gettext as _
import xl.plugins as plugins
import xl.path

PLUGIN_NAME = _("iPod Device Driver")
PLUGIN_AUTHORS = ['Adam Olsen <arolsen@gmail.com>']
PLUGIN_VERSION = '0.4.6'
PLUGIN_DESCRIPTION = _(r"""iPod Driver for the Devices Panel
\n\nRequires libgpod and its Python bindings to be installed
\n(e.g. python-gpod in Debian).""")

PLUGIN_ENABLED = False
button = gtk.Button()
PLUGIN_ICON = button.render_icon('gnome-dev-ipod', gtk.ICON_SIZE_MENU)
button.destroy()

PLUGIN = None

try:
    import gpod
    IPOD_AVAIL = True
except ImportError:
    IPOD_AVAIL = False

def configure():
    """
        Shows the configuration dialog
    """
    exaile = APP
    dialog = plugins.PluginConfigDialog(exaile.window, PLUGIN_NAME)
    table = gtk.Table(1, 2)
    table.set_row_spacings(2)
    bottom = 0
    label = gtk.Label(_("Mount Point:      "))
    label.set_alignment(0.0, 0.5)

    table.attach(label, 0, 1, bottom, bottom + 1)

    location = exaile.settings.get_str("ipod_mount", plugin=plugins.name(__file__),
        default="/media/ipod")

    loc_entry = gtk.FileChooserButton(_("Location"))
    loc_entry.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
    loc_entry.set_current_folder(location)
    table.attach(loc_entry, 1, 2, bottom, bottom + 1, gtk.SHRINK)

    dialog.child.pack_start(table)
    dialog.show_all()

    result = dialog.run()
    dialog.hide()
    if result == gtk.RESPONSE_OK:
        exaile.settings.set_str("ipod_mount", loc_entry.get_current_folder(),
            plugin=plugins.name(__file__))

class iPodTrack(media.Track):
    
    type = 'device'
    def __init__(self, *args):
        """
            Initializes the track
        """
        media.Track.__init__(self, *args)

        self.itrack = None

    def ipod_track(self):
        """
            Returns the ipod track
        """
        return self.itrack

    def get_rating(self): 
        """
            Gets the rating
        """
        # this is an approximate conversion from the iPod's rating system
        return "* " * int(self._rating / 14) 
    
    def set_rating(self, rating):
        """
            Sets the rating
        """
        self._rating = rating

    rating = property(get_rating, set_rating)

class iPodPlaylist(object):
    """
        Container for iPod playlist
    """
    def __init__(self, playlist, root=False):
        """
            requires an gpod playlist object
        """
        self.playlist = playlist
        self.name = playlist.name
        self.root = root

    def __str__(self):
        """
            Returns the playlist name
        """
        return self.playlist.name

class iPodDriver(plugins.DeviceDriver):
    name = 'ipod'
    def __init__(self):
        plugins.DeviceDriver.__init__(self)
        self.itdb = None
        self.db = None
        self.exaile = APP
        self.dp = APP.device_panel

        self.ipod_image = xlmisc.get_icon('gnome-dev-ipod')
        self.iplaylist_image = xlmisc.get_icon('gtk-justify-center')
        self.stopped = False

    def remove_tracks(self, tracks):
        """ 
            Removes tracks from the ipod
        """
        for track in tracks:
            song = track 
            track = song.itrack 
            for playlist in gpod.sw_get_playlists(self.itdb):
                if gpod.itdb_playlist_contains_track(playlist, track):
                    gpod.itdb_playlist_remove_track(playlist, track)

            gpod.itdb_track_unlink(track)
            self.all.remove(song)
            for v in self.lists.itervalues():
                try:
                    v.remove(song.loc)
                except ValueError:
                    pass

        self.transfer_done()

    def get_menu(self, item, menu):
        """
            Returns the appropriate menu for a specific item
        """
        if isinstance(item, iPodPlaylist):
            menu = xlmisc.Menu()
            menu.append(_('Add Playlist'), self.on_add_playlist)
            menu.append(_('Rename Playlist'), self.on_rename_playlist)
            menu.append(_('Remove Playlist'), self.on_remove_playlist)

        return menu

    def check_open_item(self, item):
        """
            Checks to see if this is an ipod playlist, and if it is, opens the
            playlist
        """
        if isinstance(item, iPodPlaylist):
            songs = library.TrackData()
            for path in self.lists[item]:
                song = self.all.for_path(path)
                if song:
                    songs.append(song)
            self.exaile.new_page(item.name, songs)
            self.exaile.tracks.playlist = item.name
            return True

        return False

    def on_add_playlist(self, *e): 
        """
            Creates a new playlist on the ipod
        """
        dialog = common.TextEntryDialog(self.exaile.window, 
            _("Enter a name for the new playlist"), _("New Playlist"))
        result = dialog.run()
        if result == gtk.RESPONSE_OK:
            name = str(dialog.get_value())
            playlist = gpod.itdb_playlist_new(name, False)
            p = iPodPlaylist(playlist)
            self.lists[p] = []
            gpod.itdb_playlist_add(self.itdb, playlist, -1)
            item = self.dp.model.append(self.iroot, [self.iplaylist_image,
                p, 'nofield'])
            self.lists_dict[name] = playlist
            self.transfer_done()

            self.dp.load_tree()

    def on_rename_playlist(self, *e): 
        """
            Renames a playlist on the ipod
        """
        selection = self.dp.tree.get_selection()
        (model, paths) = selection.get_selected_rows()
        for path in paths:
            iter = model.get_iter(path)
            playlist = model.get_value(iter, 1)

            dialog = common.TextEntryDialog(self.exaile.window, 
                _("Enter the new name for the playlist"), 
                _("Rename Playlist"))
            result = dialog.run()
            if result == gtk.RESPONSE_OK:
                name = str(dialog.get_value())

                if self.list_dict.has_key(playlist.name):
                    del self.list_dict[playlist.name]
                playlist.playlist.name = name
                playlist.name = name
                self.list_dict[name] = playlist
                self.transfer_done()
                self.dp.tree.queue_draw()

    def on_remove_playlist(self, *e): 
        selection = self.dp.tree.get_selection()
        (model, paths) = selection.get_selected_rows()
        for path in paths:
            iter = model.get_iter(path)
            playlist = model.get_value(iter, 1)

            dialog = gtk.MessageDialog(self.exaile.window, 
                gtk.DIALOG_MODAL, gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO, 
                _("Are you sure you want to permanently delete the selected"
                " playlist?"))
            result = dialog.run()
            dialog.destroy()
            if result == gtk.RESPONSE_YES:
                gpod.itdb_playlist_remove(playlist.playlist)
                if self.list_dict.has_key(playlist.name):
                    del self.list_dict[playlist.name]
                if self.lists.has_key(playlist):
                    del self.lists[playlist]
                self.dp.model.remove(iter) 
                self.transfer_done()
                self.dp.tree.queue_draw()
            break

    def get_initial_root(self, model):
        """
            Adds new nodes and returns the one tracks should be added to
        """
        if not self.lists: return None

        self.iroot = model.append(None, [self.ipod_image, self.root_playlist,
			'nofield'])

        for k, v in self.lists.iteritems():
            if k != self.root_playlist:
                item = model.append(self.iroot, [self.iplaylist_image,
                k, 'nofield'])
        path = model.get_path(self.iroot)
        new_root = model.append(None, [self.ipod_image, _("iPod Collection"),
			'nofield'])
        return new_root

    def done_loading_tree(self):
        """
            Called when the tree is done loading
        """
        path = self.dp.model.get_path(self.iroot)
        self.dp.tree.expand_row(path, False)

    def put_item(self, item):
        """
            Transfers a track to the ipod
        """
        (song, target) = (item.track, item.target)
        xlmisc.log("Transferring %s %s" % (song, type(song)))

        # check to see if the track is transferrable
        if self.find_dup(song): return
        if song.type != 'mp3' and song.type != 'aac': return
        if hasattr(song, 'itrack'): return

        track = self.get_ipod_track(song)
        if not track: return
        song.itrack = track
        cover = self.get_cover_location(song)
        track.itdb = self.itdb
        if cover:
            gpod.itdb_track_set_thumbnails(track, cover)

        loc = str(song.io_loc)

        gpod.itdb_cp_track_to_ipod(track, loc, None)
        gpod.itdb_track_add(self.itdb, track, -1)
        mpl = gpod.itdb_playlist_mpl(self.itdb)
        gpod.itdb_playlist_add_track(mpl, track, -1)

        if isinstance(target, iPodPlaylist):
            gpod.itdb_playlist_add_track(target.playlist, track, -1)
            if not loc in self.lists[target]:
                self.lists[target].append(loc)

        song = iPodTrack(loc, track.title, track.artist, track.album, 0,
        track.genre, track.track_nr, track.tracklen / 1000, track.bitrate,
            track.year)
        song.itrack = track
        self.all.append(song)

    def find_dup(self, song):
        """
            Finds if the song is already on the ipod
        """
        for s in self.all:
            if s.title != song.title:
                continue
            elif s.artist != song.artist:
                continue
            elif s.album != song.album:
                continue
            elif s.duration != song.duration:
                continue
            elif s.title != song.title:
                continue
            else:
                return True

        return False

    def get_ipod_track(self, song):
        """
            Returns an ipod compatible track
        """
        try:
            track = gpod.itdb_track_new()
            track.title = str(song.title)
            track.album = str(song.album)
            track.artist = str(song.artist)
            track.tracklen = song.duration * 1000

            try: track.bitrate = int(song._bitrate)
            except: pass
            try: track.track_nr = int(song.track)
            except: pass
            try: track.year = int(song.year)
            except: pass

            if song.type != 'podcast':
                info = os.stat(song.loc)
            else:
                info = os.stat(song.download_path)
            track.size = info[6]

            # libgpod >= 0.5.2 doesn't use mac-type timestamps
            try:
                track.time_added = int(time.time()) + 2082844800
            except TypeError:
                track.time_added = int(time.time())
            track.time_modified = track.time_added
            track.genre = str(song.genre)

            return track 
        except: 
            xlmisc.log_exception()
            return None

    def transfer_done(self):
        gpod.itdb_write(self.itdb, None)

    def get_cover_location(self, track):
        """
            Gets the location of the album art
        """
        db = APP.db
        
        rows = db.select("SELECT image FROM tracks,albums,paths WHERE paths.name=?"
            " AND paths.id=tracks.path AND albums.id=tracks.album",
            (track.loc,))
        if not rows: return None
        row = rows[0]
        if not row or row[0] == '': return None
        return xl.path.get_config('covers', str(row[0]))

    @common.threaded
    def connect(self, panel):
        """
            Connects to the ipod
        """
        self.stopped = False
        self.mount = self.exaile.settings.get_str("ipod_mount",
            plugin=plugins.name(__file__), default="/media/ipod")

        self.mount = str(self.mount)
        self.itdb = gpod.itdb_parse(self.mount, None)
        self.panel = panel

        self.lists = {} 
        self.lists_dict = {}
        self.all = library.TrackData()
        self.root_playlist = None

        if not self.itdb: 
            self.connected = False
            self.all = library.TrackData()
            gobject.idle_add(panel.on_error, _("Error connecting to "
                "iPod. Make sure you specify the right mount point "
                "in the plugin configuration."))
            return False

        self.all = library.TrackData()

        for playlist in gpod.sw_get_playlists(self.itdb):
            if playlist.type == 1:
                p = iPodPlaylist(playlist, True)
                self.root_playlist = p
            else:
                p = iPodPlaylist(playlist, False)

            self.lists[p] = []
            self.lists_dict[p.name] = p
            for track in gpod.sw_get_playlist_tracks(playlist):
                loc = self.mount + track.ipod_path.replace(":", "/")
                self.lists[p].append(loc)

        tracks = library.TrackData() 
        for track in gpod.sw_get_tracks(self.itdb):
            if self.stopped:
                self.stopped = False
                return
            loc = self.mount + track.ipod_path.replace(":", "/")
            loc = unicode(loc)
            song = iPodTrack(loc)
            song.itrack = track
            for item in ('title', 'bitrate', 'album', 'artist', 'year',
                'genre', 'rating'):
                if hasattr(track, item) and getattr(track, item):
                    setattr(song, item, getattr(track, item))
            
            song.track = track.track_nr
            song.length = track.tracklen / 1000
        
            tracks.append(song)

        self.all = tracks
        self.connected = True
        self.panel.on_connect_complete(self)
        
    def search_tracks(self, keyword):

        if keyword:
            check = []
            for track in self.all:
                for item in ('artist', 'album', 'title'):
                    attr = getattr(track, item)
                    if keyword.lower() in attr.lower():
                        check.append(track) 
        else:
            check = self.all
        new = [(a.artist, a.album, a.track, a.title, a) for a in check]
        new.sort()
        return library.TrackData([a[4] for a in new])
    
    def disconnect(self):
        self.stopped = True

def initialize():
    global PLUGIN

    if not IPOD_AVAIL:
        common.error(APP.window, _("python-gpod could not be loaded. iPod"
            " device driver will not be available"))
        return False
    PLUGIN = iPodDriver()
    APP.device_panel.add_driver(PLUGIN, PLUGIN_NAME)

    return True

def destroy():
    global PLUGIN

    if PLUGIN:
        APP.device_panel.remove_driver(PLUGIN)

    PLUGIN = None
